/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package account defines the function about generating account address by private key
package account

import (
	"fmt"

	"chainmaker.org/chainmaker/common/v2/crypto"
	"chainmaker.org/chainmaker/common/v2/crypto/asym"
	sdk "chainmaker.org/chainmaker/sdk-go/v2"
)

const (

	//addressTypeZXL 智信链格式地址
	addressTypeZXL = "zhixinchain"

	//addressTypeCM 长安链格式地址
	addressTypeCM = "chainmaker"

	//addressTypeEVM 以太坊格式地址
	addressTypeEVM = "ethereum"
)

// GenAccountAddress gen account address with specified format
func GenAccountAddress(addressType string, key []byte) (string, error) {

	// 解析私钥数据
	privKey, err := asym.PrivateKeyFromPEM(key, nil)
	if err != nil {
		return "", err
	}

	// 获取公钥
	pemStr, err := privKey.PublicKey().String()
	if err != nil {
		return "", err
	}

	// 根据公钥生成对应地址
	return GenAccountAddressByPubkey(addressType, pemStr)
}

// GenAccountAddressByPubkey gen account address with public key
func GenAccountAddressByPubkey(addressType string, pubkey string) (string, error) {

	// 根据指定地址格式，生成对应地址
	switch addressType {

	// 生成以太坊地址
	case addressTypeEVM:
		addrEvm, err := sdk.GetEVMAddressFromPKPEM(pubkey, crypto.HASH_TYPE_SHA256)
		if err != nil {
			return "", err
		}

		return addrEvm, nil

	// 生成智信链地址
	case addressTypeZXL:
		addrZxl, err := sdk.GetZXAddressFromPKPEM(pubkey, crypto.HASH_TYPE_SHA256)
		if err != nil {
			return "", err
		}

		return addrZxl, nil

	// 生成长安链地址
	case addressTypeCM:
		addrCm, err := sdk.GetCMAddressFromPKPEM(pubkey, crypto.HASH_TYPE_SHA256)
		if err != nil {
			return "", err
		}

		return addrCm, nil

	// 报错不支持的地址格式
	default:
		return "", fmt.Errorf("unsupported address type: %s", addressType)
	}
}
