/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package pkcs11 pkcs11 签名接口定义
package pkcs11

import (
	"fmt"
	"strings"

	"chainmaker.org/chainmaker/common/v2/cert"
	bccrypto "chainmaker.org/chainmaker/common/v2/crypto"
	"chainmaker.org/chainmaker/common/v2/crypto/asym"
	"chainmaker.org/chainmaker/common/v2/crypto/pkcs11"
	"chainmaker.org/chainmaker/common/v2/crypto/sdf"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/account"
	innerCfg "chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/config"
)

// hsmHandleMap is a global handle map for pkcs11 or sdf csp
var hsmHandleMap = map[string]interface{}{}

// getHSMHandleId 获取HSM Handle Id
func getHSMHandleId(library, label string) string {
	return library + label
}

// getHSMHandle 获取HSM Handle
func getHSMHandle(cfg innerCfg.Pkcs11Conf) (interface{}, error) {
	var err error
	// 获取 hsm 的 handle id
	hsmKey := getHSMHandleId(cfg.Library, cfg.Label)

	// 如果已经初始化过，则直接返回
	handle, ok := hsmHandleMap[hsmKey]
	if !ok {

		// 初始化 pkcs11 签名对象
		if strings.EqualFold(cfg.Type, account.Pkcs11) {
			handle, err = pkcs11.New(cfg.Library, cfg.Label, cfg.Password, cfg.SessionCacheSize,
				cfg.Hash)

			// 初始化 sdf 签名对象
		} else if strings.EqualFold(cfg.Type, account.Sdf) {
			handle, err = sdf.New(cfg.Library, cfg.SessionCacheSize)
		} else {
			err = fmt.Errorf("invalid csp type, want pkcs11 | sdf, got %s", cfg.Type)
		}
		if err != nil {
			return nil, fmt.Errorf("fail to initialize organization with HSM: [%v]", err)
		}

		// 初始化成功后缓存下
		hsmHandleMap[hsmKey] = handle
	}
	return handle, nil
}

// GetSignKey 获取签名私钥对象
// @param cfg
// @param skPEM
// @return bccrypto.PrivateKey
// @return error
func GetSignKey(cfg innerCfg.Pkcs11Conf, skPEM []byte) (sk bccrypto.PrivateKey, err error) {
	// 如果开启 pkcs11 模式，则使用 pkcs11 私钥对象
	if cfg.Enabled {
		var handle interface{}

		// 获取 pkcs11 handler
		handle, err = getHSMHandle(cfg)
		if err != nil {
			return nil, fmt.Errorf("fail to getHSMHandle: %v", err)
		}

		// 解析 pkcs11 私钥对象
		sk, err = cert.ParseP11PrivKey(handle, skPEM)
		if err != nil {
			return nil, fmt.Errorf("fail to ParseP11PrivKey: %v", err)
		}
	} else {
		// 如果不开启 pkcs11 模式, 则使用 asym 私钥对象
		sk, err = asym.PrivateKeyFromPEM(skPEM, nil)
		if err != nil {
			return nil, fmt.Errorf("fail to PrivateKeyFromPEM: %v", err)
		}
	}

	return
}
