package pkcs11

import (
	"crypto/rand"
	"encoding/hex"
	"errors"

	"chainmaker.org/chainmaker/common/v2/crypto"
	bccrypto "chainmaker.org/chainmaker/common/v2/crypto"
	"chainmaker.org/chainmaker/common/v2/crypto/pkcs11"
	internal_config "chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/config"
	_const "chainweaver.org.cn/chainweaver/mira/mira-common/crypto/const"
	"github.com/tjfoc/gmsm/sm2"
)

// PKCS11 硬件接口
type PKCS11 struct {
	*pkcs11.P11Handle
}

// NewPKCS11 初始化PKCS11
// @param cfg
func NewPKCS11(cfg *internal_config.Pkcs11Conf) (*PKCS11, error) {
	p11, err := pkcs11.New(cfg.Library, cfg.Label, cfg.Password, cfg.SessionCacheSize, cfg.Hash)
	if err != nil {
		return nil, err
	}
	pc11 := PKCS11{p11}
	return &pc11, err
}

// Enc PKCS11 加密
// @param algo
// @param key
// @param keyPwd
// @param plainText
// @param mode
func (PC11 PKCS11) Enc(algo _const.Algorithm, key string, keyPwd []byte, plainText string,
	mode string) (string, error) {
	var (
		cipherTextByte []byte
		err            error
		sm4Key         crypto.SymmetricKey
		sm2Key         bccrypto.PrivateKey
	)
	switch algo {
	case _const.SM4:
		sm4Key, err = pkcs11.NewSM4Key(PC11.P11Handle, []byte(key))
		if err != nil {
			return "", err
		}
		cipherTextByte, err = sm4Key.Encrypt([]byte(plainText))
	case _const.SM2:
		sm2Key, err = pkcs11.NewPrivateKey(PC11.P11Handle, key, bccrypto.SM2)
		if err != nil {
			return "", err
		}
		pkSM2, ok := sm2Key.PublicKey().ToStandardKey().(*sm2.PublicKey)
		if !ok {
			return "", errors.New("SM2 private key does not match the type it claims")
		}
		cipherTextByte, err = pkSM2.EncryptAsn1([]byte(plainText), rand.Reader)
	default:
		return "", errors.New("only support [SM2 | SM4] now")
	}
	if err != nil {
		return "", err
	}

	// 密文 hex 格式
	return hex.EncodeToString(cipherTextByte), nil
}

// Dec PKCS11 解密
// @param algo
// @param key
// @param keyPwd
// @param cipherText
// @param mode
func (PC11 PKCS11) Dec(algo _const.Algorithm, key string,
	keyPwd []byte, cipherText string, mode string) (string, error) {
	var (
		plainTextByte []byte
		err           error
		sm4Key        crypto.SymmetricKey
		sm2Key        bccrypto.PrivateKey
	)
	// 判断算法类型
	switch algo {
	// 如果是SM4
	case _const.SM4:
		// 获取SM4密钥
		sm4Key, err = pkcs11.NewSM4Key(PC11.P11Handle, []byte(key))
		if err != nil {
			return "", err
		}
		// 解密
		plainTextByte, err = sm4Key.Decrypt([]byte(cipherText))
	//	 如果是SM2
	case _const.SM2:
		// 获取SM2密钥
		sm2Key, err = pkcs11.NewPrivateKey(PC11.P11Handle, key, bccrypto.SM2)
		if err != nil {
			return "", err
		}
		// 转换为标准密钥
		skSM2, ok := sm2Key.ToStandardKey().(*sm2.PrivateKey)
		if !ok {
			return "", errors.New("invalid sm2 key")
		}
		// 解密
		plainTextByte, err = skSM2.DecryptAsn1([]byte(cipherText))
	default:
		return "", errors.New("only support [SM2 | SM4] now")
	}
	if err != nil {
		return "", err
	}

	// 明文 hex 格式
	return hex.EncodeToString(plainTextByte), nil
}
