/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package kms 定义外接 kms 交互接口
package kms

import (
	"encoding/base64"
	"encoding/json"
	"encoding/pem"
	"fmt"
	"net/http"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"

	"chainmaker.org/chainmaker/common/v2/crypto/x509"
	"chainmaker.org/chainmaker/pb-go/v2/accesscontrol"
	"chainmaker.org/chainmaker/pb-go/v2/common"
	"chainweaver.org.cn/chainweaver/ida/common/v2/chain"
	"github.com/gogo/protobuf/proto"
)

const (
	// 根据 keyJson 查询公钥 pem 字符串的 api path
	getPublicKey = "/kms/getPublicKey"

	// 根据 keyJson 签名的 api path
	sign = "/kms/sign"
)

// Response kms adapter 响应结构
type Response struct {
	Code int         `json:"code"`
	Msg  string      `json:"msg"`
	Data interface{} `json:"data"`
}

// Client kms adapter 客户端对象
type Client struct {
	Endpoint string
}

// GetPublicKey 获取公钥
func (c *Client) GetPublicKey(keyJson string) (string, error) {
	url := c.Endpoint + getPublicKey

	// 构造 getPublicKey 请求参数
	p := map[string]string{
		"keyJson": keyJson,
	}

	// 发送 getPublicKey 请求
	bBytes, err := util.SendHttpRequest(url, http.MethodPost, p)
	if err != nil {
		return "", fmt.Errorf("SendHttpRequest failed: %v", err)
	}

	// 解析 kms 响应
	var kmsRes Response
	err = json.Unmarshal(bBytes, &kmsRes)
	if err != nil {
		return "", fmt.Errorf("json unmarshal response body failed: %v", err)
	}

	// 判断 kms 响应是否成功
	if kmsRes.Code != 0 {
		return "", fmt.Errorf("kms getPublicKey failed: %v", kmsRes.Msg)
	}

	// 返回公钥 pem 字符串结果
	return kmsRes.Data.(map[string]interface{})["publicKey"].(string), nil
}

// Sign 签名
func (c *Client) Sign(keyJson string, raw []byte) (string, error) {
	url := c.Endpoint + sign

	// 构造 sign 请求参数
	p := map[string]string{
		"keyJson": keyJson,
		"raw":     base64.StdEncoding.EncodeToString(raw),
	}

	// 发送 sign 请求
	bBytes, err := util.SendHttpRequest(url, http.MethodPost, p)
	if err != nil {
		return "", fmt.Errorf("SendHttpRequest failed: %v", err)
	}

	// 解析 kms 响应
	var kmsRes Response
	err = json.Unmarshal(bBytes, &kmsRes)
	if err != nil {
		return "", fmt.Errorf("json unmarshal response body failed: %v", err)
	}

	// 判断 kms 响应是否成功
	if kmsRes.Code != 0 {
		return "", fmt.Errorf("kms sign failed: %v", kmsRes.Msg)
	}

	// 签名结果 base64 解码
	baseSig, ok := kmsRes.Data.(map[string]interface{})["signature"].(string)
	if !ok {
		return "", fmt.Errorf("kms sign result invalid: %v", kmsRes.Data)
	}

	rawSig, err := base64.StdEncoding.DecodeString(baseSig)
	if err != nil {
		return "", fmt.Errorf("failed to base64 decode %s,err: %v", baseSig, err)
	}

	// 返回签名结果
	return string(rawSig), nil
}

// SignTxPayload 签名交易 payload
func (c *Client) SignTxPayload(keyJson string, payload *common.Payload) (string, error) {

	// 使用 proto 序列化 tx payload，和链上签名流程一致
	payloadBytes, err := proto.Marshal(payload)
	if err != nil {
		return "", fmt.Errorf("proto marshal payload failed: %v", err)
	}

	// 签名
	return c.Sign(keyJson, payloadBytes)
}

// CreateSignerMember 获取签名者信息
func (c *Client) CreateSignerMember(mode, orgId, pkPem string,
	cert *x509.Certificate) (*accesscontrol.Member, error) {
	switch mode {

	// 构造 cert 模式的签名者信息
	case chain.KEY_MODE_CERT:
		return &accesscontrol.Member{
			OrgId:      orgId,
			MemberInfo: pem.EncodeToMemory(&pem.Block{Bytes: cert.Raw, Type: "CERTIFICATE"}),
			MemberType: accesscontrol.MemberType_CERT,
		}, nil

	// 构造 pk 模式的签名者信息
	case chain.KEY_MODE_PK:
		return &accesscontrol.Member{
			MemberInfo: []byte(pkPem),
			MemberType: accesscontrol.MemberType_PUBLIC_KEY,
		}, nil
	}

	return nil, fmt.Errorf("unsupported key mode: %s", mode)
}
