/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package logic defines all logic of key-service
package logic

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"time"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/pb-go/v2/common"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/chain"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/pb/chainpb"
	commonChain "chainweaver.org.cn/chainweaver/ida/common/v2/chain"
	commonKey "chainweaver.org.cn/chainweaver/ida/common/v2/key"
	"chainweaver.org.cn/chainweaver/ida/contract-ida/v2/types"
	"github.com/zeromicro/go-zero/core/logx"
)

// errNilChainClient chain-service 客户端为空错误
var errNilChainClient = "nil chain client"

// AccountConfigLogic defines the object to config account
type AccountConfigLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewAccountConfigLogic create the object to config account
func NewAccountConfigLogic(ctx context.Context, svcCtx *svc.ServiceContext) *AccountConfigLogic {
	return &AccountConfigLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// AccountConfig 保存链账户配置
func (l *AccountConfigLogic) AccountConfig(in *keypb.AccountConfigRequest) (*keypb.AccountConfigResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId":        in.RequestId,
		"keyId":            in.KeyId,
		"accountName":      in.AccountName,
		"socialCreditCode": in.SocialCreditCode,
		"accountTypes":     in.AccountTypes,
		"authUsers":        in.AuthUsers,
		"did":              in.Did,
	}

	// 检查 chain-service 客户端是否初始化
	if l.svcCtx.ChainClient == nil {
		fields["err"] = errNilChainClient

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrChainServiceUnReachable.String())
		return l.errorResponse(code.ErrChainServiceUnReachable, errors.New(errNilChainClient)), nil
	}

	// 先查询链账户信息
	keyInfo, _, err := l.svcCtx.KeyInfoModel.QueryByKeyId(in.KeyId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrQueryByKeyId.String())
		return l.errorResponse(code.ErrQueryByKeyId, err), nil
	}

	//// 如果该 key 有授权过，随便指定一个userId，用来去=取 key 签名上链；如果没有授权用户，说明是刚导入的 key
	//userId := int32(0)
	//for k := range authUsers {
	//	userId = int32(k)
	//	break
	//}

	// 注册 ida 链账户身份
	err = l.registerIdaUser(in.RequestId, in.AccountName, in.SocialCreditCode, in.AccountTypes,
		int32(keyInfo.ChainInfoId), fields, keyInfo.AccountName != "")
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrRegisterIdaUser.String())
		return l.errorResponse(code.ErrRegisterIdaUser, err), nil
	}

	// 更新本地链账户信息其他字段
	err = l.svcCtx.KeyInfoModel.Update(uint(in.KeyId), in.AccountName, in.SocialCreditCode,
		in.AuthUsers, in.AccountTypes, in.Did)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrUpdateKeyInfo.String())
		return l.errorResponse(code.ErrUpdateKeyInfo, err), nil
	}

	// 配置链账户逻辑执行成功返回
	l.Logger.WithFields(util.ConvertToLogFields(fields)...).Info("success to config account")
	return l.successResponse(), nil
}

// 组装合约调用参数对
func (l *AccountConfigLogic) createIdaKvs(accountName, socialCreditCode string,
	isExist bool, accountTypes []string) ([]*chainpb.KeyValuePair, error) {
	kvs := make([]*chainpb.KeyValuePair, 0)
	now := time.Now().Format(time.RFC3339)

	// 至少得已指定一个角色类型
	if len(accountTypes) == 0 {
		return nil, fmt.Errorf("empty account type")
	}

	// 根据链账户身份类型，转成合约对应的常量值
	enterpriseTypes := make([]types.EnterpriseType, 0)
	for _, at := range accountTypes {
		switch at {
		case commonKey.Registrant:
			enterpriseTypes = append(enterpriseTypes, types.EnterpriseType_Registration)
		case commonKey.Evaluator:
			enterpriseTypes = append(enterpriseTypes, types.EnterpriseType_Assessment)
		case commonKey.Reviewer:
			enterpriseTypes = append(enterpriseTypes, types.EnterpriseType_Certification)
		default:
			return nil, fmt.Errorf("unknown account type: %s", at)
		}
	}

	// 转成 json 字符串
	enterpriseTypeBytes, err := json.Marshal(enterpriseTypes)
	if err != nil {
		return nil, fmt.Errorf("failed to json marshal enterpriseTypes: %v", err)
	}

	// 组装通用的合约参数对
	kvs = append(kvs,
		&chainpb.KeyValuePair{
			Key:   "name",
			Value: []byte(accountName),
		},
		&chainpb.KeyValuePair{
			Key:   "creditCode",
			Value: []byte(socialCreditCode),
		},
		&chainpb.KeyValuePair{
			Key:   "enterpriseTypes",
			Value: enterpriseTypeBytes,
		},
	)

	// 如果是注册用户，增加 create_at 参数对
	if !isExist {
		kvs = append(kvs, &chainpb.KeyValuePair{
			Key:   "createdAt",
			Value: []byte(now),
		})
	}

	return kvs, nil
}

// errorResponse returns the error response.
func (l *AccountConfigLogic) errorResponse(code code.RespCode, err error) *keypb.AccountConfigResponse {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.AccountConfigResponse{
		Code: int32(code),
		Msg:  msg,
	}
}

// successResponse returns the success response.
func (l *AccountConfigLogic) successResponse() *keypb.AccountConfigResponse {
	return &keypb.AccountConfigResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
	}
}

// registerIdaUser 注册链账户身份
func (l *AccountConfigLogic) registerIdaUser(requestId, accountName, socialCreditCode string, accountTypes []string,
	chainInfoId int32, fields map[string]interface{}, isExist bool) error {

	// 创建链上注册所有的参数对
	//isExist := registerId != 0
	kvs, err := l.createIdaKvs(accountName, socialCreditCode, isExist, accountTypes)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrCreateKvs.String())
		return err
	}

	// 如果已经注册过，更新上链，否则新注册上链
	sync := true
	contractType := commonChain.ContractTypeIda
	cin := &chain.ChainContractRequest{
		RequestId: requestId,
		//UserId:         &userId,
		//ChainId:        &keyInfo.ChainId,
		//SubChainId:     &keyInfo.SubChainId,
		//ContractName:   &l.svcCtx.Config.ContractName,
		ContractMethod: "ModifyUser",
		KvPairs:        kvs,
		MethodType:     chainpb.MethodType_Invoke,
		ChainInfoId:    chainInfoId,
		ContractType:   &contractType,
		WithSyncResult: &sync,
	}

	// 如果是新注册，则调用 RegisterUser
	if !isExist {
		cin.ContractMethod = "RegisterUser"
	}

	// 日志中添加合约方法
	fields["contractMethod"] = cin.ContractMethod

	// 调用合约，注册或者更新上链
	resp, err := l.svcCtx.ChainClient.CallContract(l.ctx, cin)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrSendCallContactReq)
		return err
	}

	// 判断 chain-service 返回结果
	if resp.Code != int32(code.Success) {
		fields["err"] = resp.Msg

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrExecuteCallContact)
		return errors.New(resp.Msg)
	}

	// 检查链上交易返回结果是否为空
	if resp.TxResponse == nil {
		fields["err"] = code.ErrEmptyTxResponse.String()

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrEmptyTxResponse)
		return errors.New(code.ErrEmptyTxResponse.String())
	}

	// 增加日志通用信息
	fields["txId"] = resp.TxResponse.TxId
	fields["blockHeight"] = resp.TxResponse.TxBlockHeight
	fields["txTimestamp"] = time.Unix(resp.TxResponse.TxTimestamp, 0).Format(time.DateTime)

	// 判断链上交易成功与否
	if int32(resp.TxResponse.Code) != int32(common.TxStatusCode_SUCCESS) {
		msg := resp.TxResponse.Message
		if resp.TxResponse.ContractResult != nil {
			fields["contractResult"] = resp.TxResponse.ContractResult.String()
			msg = string(resp.TxResponse.ContractResult.Result)
		}
		fields["err"] = msg

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrExecuteTx)
		return errors.New(msg)
	}

	//// 如果已经注册过，registerId 不变
	//regId := int(registerId)
	//if !isExist {
	//	regId, err = strconv.Atoi(string(resp.TxResponse.ContractResult.Result))
	//	if err != nil {
	//		fields["err"] = err
	//		fields["registerId"] = string(resp.TxResponse.ContractResult.Result)
	//
	//		// 转成结构化日志信息
	//		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrConvertRegisterId)
	//		return 0, err
	//	}
	//}

	return nil
}
