/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"errors"
	"testing"
	"time"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/common/v2/crypto/sym/sm4"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/chain"
	chainMock "chainweaver.org.cn/chainweaver/ida/chain-service/v2/mock"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/pb/chainpb"
	"github.com/DATA-DOG/go-sqlmock"
	"github.com/zeromicro/go-zero/core/logx"
	"go.uber.org/mock/gomock"
)

func createSuccessAccountConfigSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)

	callResp := &chain.ChainContractResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		TxResponse: &chainpb.TxResponse{
			Code: 0,
			ContractResult: &chainpb.ContractResult{
				Code:   0,
				Result: []byte("12"),
			},
			TxId:          "0x123213414",
			TxBlockHeight: 234124,
			TxTimestamp:   234564562,
		},
	}

	chainClient.EXPECT().CallContract(context.Background(), gomock.Any()).AnyTimes().Return(callResp, nil)

	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)
	//rows2 := sqlmock.NewRows([]string{"id", "key_id", "user_id", "user_name", "created_at", "updated_at"}).
	//	AddRow(1, 1, 1, "asd", time.Now(), time.Now())
	//mock1.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnRows(rows2)
	mock1.ExpectBegin()
	mock1.ExpectExec("UPDATE `key_info`").
		WithArgs("zxtest", "0213SJDGHH32414", "REGISTRANT", "", sqlmock.AnyArg(), 1).
		WillReturnResult(sqlmock.NewResult(1, 1))
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)
	//mock1.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnError(gorm.ErrRecordNotFound)
	//mock1.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnRows(sqlmock.NewRows(nil))
	//mock1.ExpectExec("INSERT INTO `auth_user`").
	//	WithArgs(1, 1, "jack").
	//	WillReturnResult(sqlmock.NewResult(1, 1))
	mock1.ExpectCommit()

	service := &svc.ServiceContext{
		ChainClient:  chainClient,
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	return service
}

func createErrChainServiceUnReachableSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	service := &svc.ServiceContext{
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	return service
}

func createErrQueryByKeyIdSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnError(errors.New(code.ErrQueryByKeyId.String()))

	service := &svc.ServiceContext{
		ChainClient:  chainClient,
		KeyInfoModel: keyModel1,
	}
	return service
}

func createErrRegisterIdaUserSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)

	service := &svc.ServiceContext{
		ChainClient:  chainMock.NewMockChain(ctrl),
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	return service
}

func createErrUpdateKeyInfoSVC(t *testing.T) *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)

	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)

	callResp := &chain.ChainContractResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		TxResponse: &chainpb.TxResponse{
			Code: 0,
			ContractResult: &chainpb.ContractResult{
				Code:   0,
				Result: []byte("12"),
			},
			TxId:          "0x123213414",
			TxBlockHeight: 234124,
			TxTimestamp:   234564562,
		},
	}

	chainClient.EXPECT().CallContract(context.Background(), gomock.Any()).AnyTimes().Return(callResp, nil)

	mock1.ExpectBegin()
	mock1.ExpectExec("UPDATE `key_info`").WillReturnError(errors.New(code.ErrUpdateKeyInfo.String()))
	mock1.ExpectCommit()

	service := &svc.ServiceContext{
		ChainClient:  chainClient,
		KeyInfoModel: keyModel1,
	}
	return service
}

func TestAccountConfigLogic_AccountConfig(t *testing.T) {
	type (
		fields struct {
			ctx    context.Context
			svcCtx *svc.ServiceContext
			Logger logx.Logger
		}

		args struct {
			in *keypb.AccountConfigRequest
		}
	)

	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.AccountConfigResponse
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createSuccessAccountConfigSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.AccountConfigRequest{
					KeyId:            1,
					AccountName:      "zxtest",
					AccountTypes:     []string{"REGISTRANT"},
					SocialCreditCode: "0213SJDGHH32414",
					AuthUsers:        map[uint32]string{1: "jack"},
					RequestId:        "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.AccountConfigResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
		{
			name: code.ErrChainServiceUnReachable.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrChainServiceUnReachableSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.AccountConfigRequest{
					KeyId:            1,
					AccountName:      "zxtest",
					AccountTypes:     []string{"REGISTRANT"},
					SocialCreditCode: "0213SJDGHH32414",
					AuthUsers:        map[uint32]string{1: "jack"},
					RequestId:        "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.AccountConfigResponse{
				Code: int32(code.ErrChainServiceUnReachable),
				Msg:  errNilChainClient,
			},
		},
		{
			name: code.ErrQueryByKeyId.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrQueryByKeyIdSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.AccountConfigRequest{
					KeyId:            1,
					AccountName:      "zxtest",
					AccountTypes:     []string{"REGISTRANT"},
					SocialCreditCode: "0213SJDGHH32414",
					AuthUsers:        map[uint32]string{1: "jack"},
					RequestId:        "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.AccountConfigResponse{
				Code: int32(code.ErrQueryByKeyId),
				Msg:  code.ErrQueryByKeyId.String(),
			},
		},
		{
			name: code.ErrRegisterIdaUser.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrRegisterIdaUserSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.AccountConfigRequest{
					KeyId:            1,
					AccountName:      "zxtest",
					AccountTypes:     []string{"haha"},
					SocialCreditCode: "0213SJDGHH32414",
					AuthUsers:        map[uint32]string{1: "jack"},
					RequestId:        "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.AccountConfigResponse{
				Code: int32(code.ErrRegisterIdaUser),
				Msg:  "unknown account type: haha",
			},
		},
		{
			name: code.ErrUpdateKeyInfo.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrUpdateKeyInfoSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.AccountConfigRequest{
					KeyId:            1,
					AccountName:      "zxtest",
					AccountTypes:     []string{"REGISTRANT"},
					SocialCreditCode: "0213SJDGHH32414",
					AuthUsers:        map[uint32]string{1: "jack"},
					RequestId:        "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.AccountConfigResponse{
				Code: int32(code.ErrUpdateKeyInfo),
				Msg:  code.ErrUpdateKeyInfo.String(),
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &AccountConfigLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, _ := l.AccountConfig(tt.args.in)

			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}
