package logic

import (
	"context"
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/zeromicro/go-zero/core/logx"
)

// CreateKeyLogic 新建密钥结构
type CreateKeyLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewCreateKeyLogic 新建密钥
func NewCreateKeyLogic(ctx context.Context, svcCtx *svc.ServiceContext) *CreateKeyLogic {
	return &CreateKeyLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// CreateKey 生成密钥
func (l *CreateKeyLogic) CreateKey(in *keypb.KeyCreateRequest) (*keypb.KeyCreateResponse, error) {
	var key *types.SecretKey
	var err error
	// 判断算法类型
	switch in.AlgoType {
	// 如果是SM2
	case keypb.AlgoType_SM2:
		key, err = svc.SM2KeyPairCreate()
	// 如果是SM4
	case keypb.AlgoType_SM4:
		key, err = svc.GenSymKey(keypb.AlgoType_SM4, in.KeySize)
	//	如果是AES
	case keypb.AlgoType_AES:
		key, err = svc.GenSymKey(keypb.AlgoType_AES, in.KeySize)
	default:
		return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
	}

	// 生成失败
	if err != nil {
		return l.errorResponse(code.ErrGenerateKey, err)
	}
	// 插入数据库
	keyInfo, err := dbaccess.DBAccessService.AddSecretKeyReturnKey(key)
	if err != nil {
		return l.errorResponse(code.ErrInsertKey, err)
	}
	logx.Info("create key success，", "key is: ", key.SecretKeyID)

	// 成功返回密钥ID和公钥
	return l.successResponse(keyInfo.SecretKeyID, key.PublicKey), nil
}

// errorResponse returns the error response.
func (l *CreateKeyLogic) errorResponse(code code.RespCode, err error) (*keypb.KeyCreateResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.KeyCreateResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *CreateKeyLogic) successResponse(keyId string, pubKey string) *keypb.KeyCreateResponse {
	return &keypb.KeyCreateResponse{
		Code:   int32(code.Success),
		Msg:    code.Success.String(),
		KeyId:  keyId,
		PubKey: pubKey,
	}
}
