package logic

import (
	"context"
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/zeromicro/go-zero/core/logx"
)

// DecByKeKLogic defines the logic for DecByKeK.
type DecByKeKLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewDecByKeKLogic 新建DecByKeKLogic实例
func NewDecByKeKLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DecByKeKLogic {
	return &DecByKeKLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// DecByKeK 数字信封解密
func (l *DecByKeKLogic) DecByKeK(in *keypb.DataEnvelopeDecryptRequest) (*keypb.KeyDecryptResponse, error) {
	var (
		plainText string
		err       error
		key       *types.SecretKey
	)
	// 选择数字信封解密算法
	switch in.DataEnvelopAlgoType {
	// 如果是SM2SM4DataEnvelope，则使用SM2SM4DataEnvelope解密
	case keypb.DataEnvelopAlgoType_SM2SM4DataEnvelope:
		// 如果输入参数是空，则返回错误
		if in.PubKey == "" && in.KeyId == "" {
			return l.errorResponse(code.ErrParamInput, errors.New("no pubKey information"))
		}
		// 如果输入参数是PubKey，则根据PubKey获取SecretKey
		if in.PubKey != "" {
			if !util.IsAvailablePk(in.PubKey) {
				return l.errorResponse(code.ErrPKEncodeType, errors.New(code.ErrPKEncodeType.String()))
			}
			key, err = dbaccess.DBAccessService.GetSecretKeyByPubKey(in.PubKey)
			//	如果输入参数是KeyId，则根据KeyId获取SecretKey
		} else if in.KeyId != "" {
			key, err = dbaccess.DBAccessService.GetSecretKeyById(in.KeyId)
		}
		// 如果SecretKey为空，则返回错误
		if key == nil || err != nil {
			return l.errorResponse(code.ErrDidNotFindSk, errors.New(code.ErrDidNotFindSk.String()))
		}
		sk := key.PrivateKey
		// 解密
		plainText, err = svc.DecByKeK(sk, in.CipherText, in.Kek)
		if err != nil {
			return l.errorResponse(code.ErrDecryptMsg, err)
		}
		logx.Info("decrypt with SM2SM4DataEnvelope", "plain text: ", plainText)

		// 返回解密结果
		return l.successResponse(plainText), nil
	//	如果是SM2CPABEDataEnvelope，则使用SM2CPABEDataEnvelope解密
	case keypb.DataEnvelopAlgoType_Sm2CpAbeDataEnvelope:
		logx.Info(code.ErrNotImpl.String())
		return l.errorResponse(code.ErrNotImpl, errors.New(code.ErrNotImpl.String()))
	//	默认返回
	default:
		return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
	}
}

// errorResponse returns the error response.
func (l *DecByKeKLogic) errorResponse(code code.RespCode, err error) (*keypb.KeyDecryptResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.KeyDecryptResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *DecByKeKLogic) successResponse(plainText string) *keypb.KeyDecryptResponse {
	return &keypb.KeyDecryptResponse{
		Code:      int32(code.Success),
		Msg:       code.Success.String(),
		PlainText: plainText,
	}
}
