package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/golang/mock/gomock"
	"github.com/zeromicro/go-zero/core/logx"
)

// 测试DecByKeKLogic_DecByKeK
func TestDecByKeKLogic_DecByKeK(t *testing.T) {
	ctl := gomock.NewController(t)
	defer ctl.Finish()
	key := &types.SecretKey{
		Scenario:   "SM2SM4DataEnvelope",
		PublicKey:  "043e49a7a458869e6f2aef4e202eac8ab6653195e1091a0659bd47eb3c830faaf8699c4912c73c67fd0b0d15c7515b9ac067ccabc2685fadab04c1a7a5133d2938",
		PrivateKey: "bced69f17cc101fc65bb61a9f8e12f57bd3e2dfff5bb413a46a846b72dab7aa4",
		Status:     0,
	}
	mockDBAccessInterface := dbaccess.NewMockDBAccessInterface(ctl)
	gomock.InOrder(
		mockDBAccessInterface.EXPECT().GetSecretKeyByPubKey(key.PublicKey).Return(key, nil),
	)
	dbaccess.DBAccessService = mockDBAccessInterface

	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		in *keypb.DataEnvelopeDecryptRequest
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.KeyDecryptResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createEncryptSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.DataEnvelopeDecryptRequest{
					DataEnvelopAlgoType: keypb.DataEnvelopAlgoType_SM2SM4DataEnvelope,
					CipherText:          "3f3cb43c55a6daa136063bf1872b87a7",
					PubKey:              key.PublicKey,
					Kek:                 "04803aecd1b9dac4598d6e40a3fd7aa6c6f7caeac6586198451e079359d6a8f28473b211437a9760202d903f45a8764c8c7319750a923c180de56caeeebbb7fc3147f06149fa35bce60bc3fbf21c0b66e8862aa3c85cbe28999cb933cd979e69d95ef5930775064dc375ad0f476f7aa685",
				},
			},
			want: &keypb.KeyDecryptResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		}}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &DecByKeKLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.DecByKeK(tt.args.in)
			if (err != nil) != tt.wantErr {
				t.Errorf("DecByKeK() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试DecByKeKLogic_errorResponse
func TestDecByKeKLogic_errorResponse(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		code code.RespCode
		err  error
	}

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				code: code.ErrDidNotFindSk,
				err:  errors.New(code.ErrDidNotFindSk.String()),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &DecByKeKLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.errorResponse(tt.args.code, tt.args.err)
			if err == nil {
				t.Errorf("errorResponse() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试NewDecByKeKLogic
func TestNewDecByKeKLogic(t *testing.T) {
	type args struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
	}
	tests := []struct {
		name string
		args args
		want *CreateKeyLogic
	}{
		{
			name: "success",
			args: args{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := NewDecByKeKLogic(tt.args.ctx, tt.args.svcCtx); got == nil {
				t.Errorf("NewCreateKeyLogic() = %v, want %v", got, tt.want)
			}
		})
	}
}
