package logic

import (
	"context"
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/csp"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/zeromicro/go-zero/core/logx"
)

// DecryptLogic struct
type DecryptLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewDecryptLogic new a DecryptLogic
func NewDecryptLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DecryptLogic {
	return &DecryptLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// Decrypt 解密
func (l *DecryptLogic) Decrypt(in *keypb.KeyDecryptRequest) (*keypb.KeyDecryptResponse, error) {
	var plainText string
	// 查询算法类型
	algoType, sk, err := l.queryKeyTypeAndSk(in)
	if err != nil {
		return l.errorResponse(code.ErrQueryByKeyId, err)
	}
	// 初始化硬件或者软件加密类型
	crypto, err := csp.InitCrypto(&l.svcCtx.Config)
	if err != nil {
		return l.errorResponse(code.ErrInitCrypto, err)
	}
	// 如果传入算法类型为默认，则使用公钥查询算法类型
	if in.AlgoType != keypb.AlgoType_DEFAULT {
		switch in.AlgoType {
		// 如果是SM2
		case keypb.AlgoType_SM2:
			plainText, err = crypto.Dec("SM2", sk, nil, in.CipherText, "C1C2C3")
		//	如果是SM4
		case keypb.AlgoType_SM4:
			plainText, err = crypto.Dec("SM4", sk, nil, in.CipherText, "C1C2C3")
		//	如果是AES
		case keypb.AlgoType_AES:
			logx.Info(code.ErrNotImpl.String())
			return l.errorResponse(code.ErrNotImpl, errors.New(code.ErrNotImpl.String()))
		default:
			return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
		}
		//	传入了算法类型
	} else if algoType != "" {
		switch algoType {
		// 如果是SM2
		case keypb.AlgoType_SM2.String():
			plainText, err = crypto.Dec("SM2", sk, nil, in.CipherText, "C1C2C3")
		//	如果是SM4
		case keypb.AlgoType_SM4.String():
			plainText, err = crypto.Dec("SM4", sk, nil, in.CipherText, "C1C2C3")
		//	如果是AES
		case keypb.AlgoType_AES.String():
			logx.Info(code.ErrNotImpl.String())
			return l.errorResponse(code.ErrNotImpl, errors.New(code.ErrNotImpl.String()))
		//	默认返回算法类型不支持
		default:
			return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
		}
	}

	// 解密失败
	if err != nil {
		return l.errorResponse(code.ErrDecryptMsg, err)
	}
	logx.Info("decrypt success,", "plain text", plainText)

	// 返回解密结果
	return l.successResponse(plainText), nil
}

// 根据公钥查询算法类型 returns the error response.
func (l *DecryptLogic) queryKeyTypeAndSk(in *keypb.KeyDecryptRequest) (string, string, error) {
	var key *types.SecretKey
	// 如果输入参数为空
	if in.PubKey == "" && in.KeyId == "" {
		return "", "", errors.New("no pubKey information")
	}
	// 如果输入参数为公钥，根据公钥查询key
	if in.PubKey != "" {
		if !util.IsAvailablePk(in.PubKey) {
			return "", "", errors.New("PK encoding type is not hex")
		}
		key, _ = dbaccess.DBAccessService.GetSecretKeyByPubKey(in.PubKey)
	}
	// 如果输入参数为keyId，根据keyId查询key
	if in.KeyId != "" {
		key, _ = dbaccess.DBAccessService.GetSecretKeyById(in.KeyId)
	}
	if key == nil {
		return "", "", errors.New(code.ErrDidNotFindSk.String())
	}
	// 返回算法类型和私钥
	return key.Scenario, key.PrivateKey, nil
}

// errorResponse returns the error response.
func (l *DecryptLogic) errorResponse(code code.RespCode, err error) (*keypb.KeyDecryptResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.KeyDecryptResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *DecryptLogic) successResponse(plainText string) *keypb.KeyDecryptResponse {
	return &keypb.KeyDecryptResponse{
		Code:      int32(code.Success),
		Msg:       code.Success.String(),
		PlainText: plainText,
	}
}
