package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/golang/mock/gomock"
	"github.com/zeromicro/go-zero/core/logx"
)

// 测试 Decrypt
func TestDecryptLogic_Decrypt(t *testing.T) {
	ctl := gomock.NewController(t)
	defer ctl.Finish()
	key := &types.SecretKey{
		Scenario:   "SM2",
		PublicKey:  "04c4fc8b0e428c7157c72f374a6a0306f2146ed30793ae6be6ab1d2f1b01467a6c84af7c5c105d50735e0ed560ae34826a8a2e5105a13490b5f2f4ca0efd0410fc",
		PrivateKey: "ebe1d9ad102d9465f94e86e8479eae811b83bf6204d778bf4670718fa188d236",
		Status:     0,
	}
	mockDBAccessInterface := dbaccess.NewMockDBAccessInterface(ctl)
	gomock.InOrder(
		mockDBAccessInterface.EXPECT().GetSecretKeyByPubKey(key.PublicKey).Return(key, nil),
	)
	dbaccess.DBAccessService = mockDBAccessInterface

	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		in *keypb.KeyDecryptRequest
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.KeyDecryptResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createEncryptSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.KeyDecryptRequest{
					AlgoType:   keypb.AlgoType_SM2,
					CipherText: "0417d91ede47c8cbadadf824ac9bd206406b98242bef6fa6fe2a44abbdcdf72b61fc98f2412efb32d09fe11cb075e1d5f2840711dda653b2fb5682684fa1cd2a02e9e6c7664792519956419b07a3c124f2d9dd533b37e727948a912663dc5c8491383878f37ac545",
					PubKey:     key.PublicKey,
				},
			},
			want: &keypb.KeyDecryptResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &DecryptLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.Decrypt(tt.args.in)
			if (err != nil) != tt.wantErr {
				t.Errorf("Decrypt() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试 errorResponse
func TestDecryptLogic_errorResponse(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		code code.RespCode
		err  error
	}

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				code: code.ErrDidNotFindSk,
				err:  errors.New(code.ErrDidNotFindSk.String()),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &DecryptLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.errorResponse(tt.args.code, tt.args.err)
			if err == nil {
				t.Errorf("errorResponse() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试 NewDecryptLogic
func TestNewDecryptLogic(t *testing.T) {
	type args struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
	}
	tests := []struct {
		name string
		args args
		want *CreateKeyLogic
	}{
		{
			name: "success",
			args: args{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := NewDecryptLogic(tt.args.ctx, tt.args.svcCtx); got == nil {
				t.Errorf("NewCreateKeyLogic() = %v, want %v", got, tt.want)
			}
		})
	}
}
