/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"github.com/DATA-DOG/go-sqlmock"
	"github.com/zeromicro/go-zero/core/logx"
)

func createSuccessDeleteKeyAndCertSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectBegin()
	//mock1.ExpectExec("DELETE FROM `auth_user`").
	//	WithArgs(1).
	//	WillReturnResult(sqlmock.NewResult(1, 1))
	mock1.ExpectExec("UPDATE `key_info`").
		WithArgs(sqlmock.AnyArg(), 1).
		WillReturnResult(sqlmock.NewResult(1, 1))
	mock1.ExpectCommit()

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func createFailedDeleteKeyAndCertSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectBegin()
	mock1.ExpectExec("UPDATE `key_info`").
		WithArgs(sqlmock.AnyArg(), 1).
		WillReturnError(errors.New("delete failed"))
	mock1.ExpectCommit()

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func TestDeleteKeyAndCertLogic_DeleteKeyAndCert(t *testing.T) {
	type (
		fields struct {
			ctx    context.Context
			svcCtx *svc.ServiceContext
			Logger logx.Logger
		}

		args struct {
			in *keypb.DeleteKeyAndCertRequest
		}
	)

	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.DeleteKeyAndCertResponse
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createSuccessDeleteKeyAndCertSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.DeleteKeyAndCertRequest{
					KeyId:     1,
					RequestId: "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.DeleteKeyAndCertResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
		{
			name: "failed to delete key info by id",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createFailedDeleteKeyAndCertSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.DeleteKeyAndCertRequest{
					KeyId:     1,
					RequestId: "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.DeleteKeyAndCertResponse{
				Code: int32(code.ErrDeleteByKeyId),
				Msg:  "delete failed",
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &DeleteKeyAndCertLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, _ := l.DeleteKeyAndCert(tt.args.in)

			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("DeleteKeyAndCert got = %v, want %v", got, tt.want)
			}
		})
	}
}
