package logic

import (
	"context"
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/csp"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/config"
	"github.com/zeromicro/go-zero/core/logx"
)

// EncryptLogic 加密密钥结构
type EncryptLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
	config.DBConfig
}

// NewEncryptLogic 新建加密
func NewEncryptLogic(ctx context.Context, svcCtx *svc.ServiceContext) *EncryptLogic {
	return &EncryptLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// Encrypt 加密
func (l *EncryptLogic) Encrypt(in *keypb.KeyEncryptRequest) (*keypb.KeyEncryptResponse, error) {
	var cipherText string
	// 初始化算法类型，硬件或者软件加密
	crypto, err := csp.InitCrypto(&l.svcCtx.Config)
	if err != nil {
		return l.errorResponse(code.ErrInitCrypto, err)
	}
	// 选择算法类型
	switch in.AlgoType {
	// 使用SM2算法进行加密
	case keypb.AlgoType_SM2:
		// 使用公钥进行加密, SM2加解密有两种模式：C1C2C3和C1C3C2，这里使用C1C2C3（国密标准推荐使用C1C2C3）
		cipherText, err = crypto.Enc("SM2", in.PubKey, nil, in.PlainText, "C1C2C3")
		if err != nil {
			return l.errorResponse(code.ErrEncryptMsg, err)
		}
		logx.Info("encrypt with SM2", "cipher text", cipherText)
	//	使用SM4算法进行加密
	case keypb.AlgoType_SM4:
		// 加密
		cipherText, err = crypto.Enc("SM4", in.PubKey, nil, in.PlainText, "C1C2C3")
		if err != nil {
			return l.errorResponse(code.ErrEncryptMsg, err)
		}
		logx.Info("encrypt with SM4", "cipher text", cipherText)
	//	使用AES算法进行加密
	case keypb.AlgoType_AES:
		// 目前暂未支持
		logx.Info(code.ErrNotImpl.String())
		return l.errorResponse(code.ErrNotImpl, errors.New(code.ErrNotImpl.String()))
	//	默认返回不支持算法类型
	default:
		return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
	}

	// 返回加密结果
	return l.successResponse(cipherText), nil
}

// errorResponse returns the error response.
func (l *EncryptLogic) errorResponse(code code.RespCode, err error) (*keypb.KeyEncryptResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.KeyEncryptResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *EncryptLogic) successResponse(cipherText string) *keypb.KeyEncryptResponse {
	return &keypb.KeyEncryptResponse{
		Code:       int32(code.Success),
		Msg:        code.Success.String(),
		CipherText: cipherText,
	}
}
