package logic

import (
	"context"
	"errors"
	"fmt"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/common/v2/crypto/sym/sm4"
	"chainweaver.org.cn/chainweaver/mira/mira-common/config"
	"github.com/zeromicro/go-zero/core/logx"
)

// 初始化
func createEncryptSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	service := &svc.ServiceContext{
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	service.Config.CryptoConfig.Soft = true
	return service
}

// 测试加密
func TestEncryptLogic_Encrypt(t *testing.T) {
	type fields struct {
		ctx      context.Context
		svcCtx   *svc.ServiceContext
		Logger   logx.Logger
		DBConfig config.DBConfig
	}
	type args struct {
		in *keypb.KeyEncryptRequest
	}
	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.KeyEncryptResponse
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createEncryptSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.KeyEncryptRequest{
					AlgoType:  keypb.AlgoType_SM2,
					PlainText: "['Test001']",
					PubKey:    "04a1a503e0ac24b7b9d550b2f284e41a8214893145a1320a770a1ef5c30f7a69534df537a9bc2ba1cfd4d9f2d8842938375022f8cbdc6768582254e1a73b03044e",
				},
			},
			want: &keypb.KeyEncryptResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &EncryptLogic{
				ctx:      tt.fields.ctx,
				svcCtx:   tt.fields.svcCtx,
				Logger:   tt.fields.Logger,
				DBConfig: tt.fields.DBConfig,
			}
			got, err := l.Encrypt(tt.args.in)
			if err != nil {
				t.Errorf("Encrypt() error = %v,", err)
				return
			}
			fmt.Println(got)
			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试错误响应
func TestEncryptLogic_errorResponse(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		code code.RespCode
		err  error
	}

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				code: code.ErrDidNotFindSk,
				err:  errors.New(code.ErrDidNotFindSk.String()),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &EncryptLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.errorResponse(tt.args.code, tt.args.err)
			if err == nil {
				t.Errorf("errorResponse() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试创建逻辑
func TestNewEncryptLogic(t *testing.T) {
	type args struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
	}
	tests := []struct {
		name string
		args args
		want *CreateKeyLogic
	}{
		{
			name: "success",
			args: args{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := NewEncryptLogic(tt.args.ctx, tt.args.svcCtx); got == nil {
				t.Errorf("NewCreateKeyLogic() = %v, want %v", got, tt.want)
			}
		})
	}
}
