package logic

import (
	"context"
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/csp"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"github.com/zeromicro/go-zero/core/logx"
)

// EncryptWithPkListLogic 密钥结构
type EncryptWithPkListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewEncryptWithPkListLogic 新建密钥
func NewEncryptWithPkListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *EncryptWithPkListLogic {
	return &EncryptWithPkListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// EncryptWithPkList 使用公钥列表加密
func (l *EncryptWithPkListLogic) EncryptWithPkList(
	in *keypb.KeyEncryptWithPkListRequest,
) (*keypb.KeyEncryptWithPkListResponse, error) {
	var cipherTextList []string
	// 初始化算法类型，硬件或者软件加密
	crypto, err := csp.InitCrypto(&l.svcCtx.Config)
	if err != nil {
		return l.errorResponse(code.ErrInitCrypto, err)
	}
	// 选择算法类型
	switch in.AlgoType {
	// 如果是SM2
	case keypb.AlgoType_SM2:
		// 遍历每个公钥
		for _, pk := range in.PubKeyList {
			// 加密
			ciphertext, err := crypto.Enc("SM2", pk, nil, in.PlainText, "C1C2C3")
			if err != nil {
				return l.errorResponse(code.ErrEncryptMsg, err)
			}
			cipherTextList = append(cipherTextList, ciphertext)
			logx.Info("encrypt with SM2", "pk", "cipher text", pk, ciphertext)
		}

		// 返回加密结果
		return l.successResponse(cipherTextList), nil
	//	暂时不支持SM4
	case keypb.AlgoType_SM4:
		logx.Info("create key with SM4")
	//	默认返回不支持算法类型
	default:
		return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
	}

	// 返回空结果
	return &keypb.KeyEncryptWithPkListResponse{}, nil
}

// errorResponse returns the error response.
func (l *EncryptWithPkListLogic) errorResponse(
	code code.RespCode,
	err error,
) (*keypb.KeyEncryptWithPkListResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.KeyEncryptWithPkListResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *EncryptWithPkListLogic) successResponse(cipherText []string) *keypb.KeyEncryptWithPkListResponse {
	return &keypb.KeyEncryptWithPkListResponse{
		Code:           int32(code.Success),
		Msg:            code.Success.String(),
		CipherTextList: cipherText,
	}
}
