package logic

import (
	"context"
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"github.com/zeromicro/go-zero/core/logx"
)

// EncWithDeKLogic 数字信封加密逻辑
type EncWithDeKLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewEncWithDeKLogic 构造函数
func NewEncWithDeKLogic(ctx context.Context, svcCtx *svc.ServiceContext) *EncWithDeKLogic {
	return &EncWithDeKLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// EncWithDeK 数字信封加密
func (l *EncWithDeKLogic) EncWithDeK(in *keypb.DataEnvelopeEncryptRequest) (*keypb.DataEnvelopeEncryptResponse, error) {
	// 选择加密算法类型
	switch in.DataEnvelopAlgoType {
	// 如果是SM2SM4DataEnvelope
	case keypb.DataEnvelopAlgoType_SM2SM4DataEnvelope:
		// 调用SM2SM4DataEnvelope加密
		ciphertext, kek, err := svc.EncWithDeK(in.PubKey, in.PlainText)
		if err != nil {
			return l.errorResponse(code.ErrEncryptMsg, err)
		}
		logx.Info("encrypt with SM2SM4DataEnvelope", "cipher text: ", ciphertext)

		// 返回加密结果
		return l.successResponse(ciphertext, kek), nil
	//	如果是SM2CPABEDataEnvelope
	case keypb.DataEnvelopAlgoType_Sm2CpAbeDataEnvelope:
		// 暂未支持
		logx.Info(code.ErrNotImpl.String())
		return l.errorResponse(code.ErrNotImpl, errors.New(code.ErrNotImpl.String()))
	//	默认返回不支持算法
	default:
		return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
	}
}

// errorResponse returns the error response.
func (l *EncWithDeKLogic) errorResponse(code code.RespCode, err error) (*keypb.DataEnvelopeEncryptResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.DataEnvelopeEncryptResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *EncWithDeKLogic) successResponse(cipherText string, kek string) *keypb.DataEnvelopeEncryptResponse {
	return &keypb.DataEnvelopeEncryptResponse{
		Code:       int32(code.Success),
		Msg:        code.Success.String(),
		CipherText: cipherText,
		Kek:        kek,
	}
}
