package logic

import (
	"context"
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"github.com/zeromicro/go-zero/core/logx"
)

// EncWithDeKWithPkListLogic 数字信封结构题
type EncWithDeKWithPkListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewEncWithDeKWithPkListLogic 数字信封逻辑
func NewEncWithDeKWithPkListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *EncWithDeKWithPkListLogic {
	return &EncWithDeKWithPkListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// EncWithDeKWithPkList 数字信封传入多个公钥，对同一个明文加密
func (l *EncWithDeKWithPkListLogic) EncWithDeKWithPkList(
	in *keypb.DataEnvelopeEncryptWithPkListRequest) (
	*keypb.DataEnvelopeEncryptWithPkListResponse, error) {
	// 判断算法类型
	switch in.DataEnvelopAlgoType {
	// 如果是SM2SM4DataEnvelope
	case keypb.DataEnvelopAlgoType_SM2SM4DataEnvelope:
		// 根据公钥列表，加密明文
		cipherText, kekList, err := svc.EncWithDekList(in.PubKeyList, in.PlainText)
		if err != nil {
			return l.errorResponse(code.ErrEncryptMsg, err)
		}
		logx.Info("encrypt with SM2SM4DataEnvelope", "cipher text: ", "KEKList :", cipherText, kekList)

		// 成功返回加密结果
		return l.successResponse(cipherText, kekList), nil
	//	如果是SM2CPABEDataEnvelope
	case keypb.DataEnvelopAlgoType_Sm2CpAbeDataEnvelope:
		// 暂未支持
		logx.Info(code.ErrNotImpl.String())
		return l.errorResponse(code.ErrNotImpl, errors.New(code.ErrNotImpl.String()))
	//	默认返回不支持类型
	default:
		return l.errorResponse(code.InvalidAlgo, errors.New(code.InvalidAlgo.String()))
	}
}

// errorResponse returns the error response.
func (l *EncWithDeKWithPkListLogic) errorResponse(
	code code.RespCode,
	err error) (*keypb.DataEnvelopeEncryptWithPkListResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}
	return &keypb.DataEnvelopeEncryptWithPkListResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *EncWithDeKWithPkListLogic) successResponse(
	cipherText string,
	kek []string) *keypb.DataEnvelopeEncryptWithPkListResponse {
	return &keypb.DataEnvelopeEncryptWithPkListResponse{
		Code:       int32(code.Success),
		Msg:        code.Success.String(),
		CipherText: cipherText,
		KekList:    kek,
	}
}
