/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/model/table"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"github.com/zeromicro/go-zero/core/logx"
)

// GetAccountInfoLogic defines the object to get account info
type GetAccountInfoLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewGetAccountInfoLogic create the object to get account info
func NewGetAccountInfoLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetAccountInfoLogic {
	return &GetAccountInfoLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// GetAccountInfo 查询区块链信息
func (l *GetAccountInfoLogic) GetAccountInfo(
	in *keypb.GetAccountInfoRequest) (*keypb.GetAccountInfoResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId":   in.RequestId,
		"chainInfoId": in.ChainInfoId,
	}

	// 取出本地的链账户（当前本地只允许拥有一个链账户）
	info, err := l.svcCtx.KeyInfoModel.QueryByChainInfoId(in.ChainInfoId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrQueryByChainInfoId.String())
		return &keypb.GetAccountInfoResponse{
			Code: int32(code.ErrQueryByChainInfoId),
			Msg:  err.Error(),
		}, nil
	}

	// gorm find 会过滤掉 RecordNotFound 错误，即没找到还是会返回一个非 nil 的对象，只不过里面的值都是默认零值
	if info.ID == 0 {

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrNotFoundKey.String())
		return &keypb.GetAccountInfoResponse{
			Code: int32(code.ErrNotFoundKey),
			Msg:  code.ErrNotFoundKey.String(),
		}, nil
	}

	// 成功返回
	return &keypb.GetAccountInfoResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),

		// 转成 response data 结构
		Data: util.ConvertToAccountData([]table.KeyInfo{*info}, nil)[0],
	}, nil
}
