/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"errors"
	"testing"
	"time"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"github.com/DATA-DOG/go-sqlmock"
	"github.com/zeromicro/go-zero/core/logx"
)

func createSuccessGetAccountInfoSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)
	//rows2 := sqlmock.NewRows([]string{"id", "key_id", "user_id", "user_name", "created_at", "updated_at"}).
	//	AddRow(1, 1, 1, "asd", time.Now(), time.Now())
	//mock1.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnRows(rows2)

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func createErrQueryByChainInfoIdSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnError(errors.New(code.ErrQueryByChainInfoId.String()))

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func createErrNotFoundKeySVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(sqlmock.NewRows(nil))

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func TestGetAccountInfoLogic_GetAccountInfo(t *testing.T) {
	type (
		fields struct {
			ctx    context.Context
			svcCtx *svc.ServiceContext
			Logger logx.Logger
		}

		args struct {
			in *keypb.GetAccountInfoRequest
		}
	)

	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.GetAccountInfoResponse
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createSuccessGetAccountInfoSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetAccountInfoRequest{
					RequestId: "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.GetAccountInfoResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
		{
			name: code.ErrQueryByChainInfoId.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrQueryByChainInfoIdSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetAccountInfoRequest{
					RequestId: "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.GetAccountInfoResponse{
				Code: int32(code.ErrQueryByChainInfoId),
				Msg:  code.ErrQueryByChainInfoId.String(),
			},
		},
		{
			name: code.ErrNotFoundKey.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrNotFoundKeySVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetAccountInfoRequest{
					RequestId: "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.GetAccountInfoResponse{
				Code: int32(code.ErrNotFoundKey),
				Msg:  code.ErrNotFoundKey.String(),
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &GetAccountInfoLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, _ := l.GetAccountInfo(tt.args.in)

			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("GetAccountInfo got = %v, want %v", got, tt.want)
			}
		})
	}
}
