package logic

import (
	"context"
	"errors"
	"time"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/pb-go/v2/common"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/pb/chainpb"
	"chainweaver.org.cn/chainweaver/ida/common/v2/chain"
	"github.com/zeromicro/go-zero/core/logx"
)

// GetDidInfoLogic defines the object to get did info
type GetDidInfoLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewGetDidInfoLogic create the object to get did info
func NewGetDidInfoLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetDidInfoLogic {
	return &GetDidInfoLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// GetDidInfo 获取 did 信息
func (l *GetDidInfoLogic) GetDidInfo(in *keypb.GetDidInfoRequest) (*keypb.GetDidInfoResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId":   in.RequestId,
		"chainInfoId": in.ChainInfoId,
		"address":     in.Address,
	}

	// 检查 chain-service 客户端是否初始化
	if l.svcCtx.ChainClient == nil {
		fields["err"] = errNilChainClient

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrChainServiceUnReachable.String())
		return l.errorResponse(code.ErrChainServiceUnReachable, errors.New(errNilChainClient)), nil
	}

	// 调用 did 合约
	contractType := chain.ContractTypeDid
	cin := chainpb.ChainContractRequest{
		RequestId:      in.RequestId,
		ContractMethod: "GetDidByAddress",
		KvPairs: []*chainpb.KeyValuePair{
			{
				Key:   "address",
				Value: []byte(in.Address),
			},
		},
		MethodType:   chainpb.MethodType_Query,
		ContractType: &contractType,
		ChainInfoId:  int32(in.ChainInfoId),
	}

	// 增加日志通用信息
	fields["contractMethod"] = "GetDidByAddress"
	fields["kvs"] = cin.KvPairs

	// 请求 chain-service 调用合约
	resp, err := l.svcCtx.ChainClient.CallContract(l.ctx, &cin)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrSendCallContactReq)
		return l.errorResponse(code.ErrSendCallContactReq, err), nil
	}

	// 判断 chain-service 返回结果
	if resp.Code != int32(code.Success) {
		fields["err"] = resp.Msg

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrExecuteCallContact)
		return l.errorResponse(code.ErrExecuteCallContact, errors.New(resp.Msg)), nil
	}

	// 检查链上交易返回结果是否为空
	if resp.TxResponse == nil {
		fields["err"] = code.ErrEmptyTxResponse.String()

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrEmptyTxResponse)
		return l.errorResponse(code.ErrEmptyTxResponse, nil), nil
	}

	// 增加日志通用信息
	fields["txId"] = resp.TxResponse.TxId
	fields["blockHeight"] = resp.TxResponse.TxBlockHeight
	fields["txTimestamp"] = time.Unix(resp.TxResponse.TxTimestamp, 0).Format(time.DateTime)

	// 判断链上交易成功与否
	if int32(resp.TxResponse.Code) != int32(common.TxStatusCode_SUCCESS) {
		msg := resp.TxResponse.Message
		if resp.TxResponse.ContractResult != nil {
			fields["contractResult"] = resp.TxResponse.ContractResult.String()
			msg = string(resp.TxResponse.ContractResult.Result)
		}
		fields["err"] = msg

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrExecuteTx)
		return l.errorResponse(code.ErrExecuteTx, errors.New(msg)), nil
	}

	// 成功返回
	return l.successResponse(&keypb.DidInfoData{Did: string(resp.TxResponse.ContractResult.Result)}), nil
}

// errorResponse returns the error response.
func (l *GetDidInfoLogic) errorResponse(code code.RespCode, err error) *keypb.GetDidInfoResponse {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.GetDidInfoResponse{
		Code: int32(code),
		Msg:  msg,
	}
}

// successResponse returns the success response.
func (l *GetDidInfoLogic) successResponse(data *keypb.DidInfoData) *keypb.GetDidInfoResponse {
	return &keypb.GetDidInfoResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		Data: data,
	}
}
