/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/chain"
	chainMock "chainweaver.org.cn/chainweaver/ida/chain-service/v2/mock"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/pb/chainpb"
	"github.com/zeromicro/go-zero/core/logx"
	"go.uber.org/mock/gomock"
)

var (
	did = "123"
)

func createSuccessGetDidInfoSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)

	callResp := &chain.ChainContractResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		TxResponse: &chainpb.TxResponse{
			Code: 0,
			ContractResult: &chainpb.ContractResult{
				Code:   0,
				Result: []byte(did),
			},
			TxId:          "0x123213414",
			TxBlockHeight: 234124,
			TxTimestamp:   234564562,
		},
	}

	chainClient.EXPECT().CallContract(context.Background(), gomock.Any()).AnyTimes().Return(callResp, nil)

	service := &svc.ServiceContext{
		ChainClient: chainClient,
	}
	return service
}

func createErrSendCallContactReqSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)
	chainClient.EXPECT().CallContract(context.Background(), gomock.Any()).AnyTimes().Return(nil, errors.New(code.ErrSendCallContactReq.String()))

	service := &svc.ServiceContext{
		ChainClient: chainClient,
	}
	return service
}

func createErrExecuteCallContactSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)

	callResp := &chain.ChainContractResponse{
		Code: -1,
		Msg:  code.ErrExecuteCallContact.String(),
	}

	chainClient.EXPECT().CallContract(context.Background(), gomock.Any()).AnyTimes().Return(callResp, nil)

	service := &svc.ServiceContext{
		ChainClient: chainClient,
	}
	return service
}

func createErrEmptyTxResponseSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)

	callResp := &chain.ChainContractResponse{
		Code:       int32(code.Success),
		Msg:        code.Success.String(),
		TxResponse: nil,
	}

	chainClient.EXPECT().CallContract(context.Background(), gomock.Any()).AnyTimes().Return(callResp, nil)

	service := &svc.ServiceContext{
		ChainClient: chainClient,
	}
	return service
}

func createErrExecuteTxSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	chainClient := chainMock.NewMockChain(ctrl)

	callResp := &chain.ChainContractResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		TxResponse: &chainpb.TxResponse{
			Code:    -1,
			Message: code.ErrExecuteTx.String(),
		},
	}

	chainClient.EXPECT().CallContract(context.Background(), gomock.Any()).AnyTimes().Return(callResp, nil)

	service := &svc.ServiceContext{
		ChainClient: chainClient,
	}
	return service
}

func TestGetDidInfoLogic_GetDidInfo(t *testing.T) {
	type (
		fields struct {
			ctx    context.Context
			svcCtx *svc.ServiceContext
			Logger logx.Logger
		}

		args struct {
			in *keypb.GetDidInfoRequest
		}
	)

	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.GetDidInfoResponse
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createSuccessGetDidInfoSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetDidInfoRequest{
					RequestId:   "asdasd-23234-fbzdfbv",
					ChainInfoId: 1,
					Address:     addr,
				},
			},
			want: &keypb.GetDidInfoResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
				Data: &keypb.DidInfoData{
					Did: did,
				},
			},
		},
		{
			name: code.ErrChainServiceUnReachable.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrChainServiceUnReachableSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetDidInfoRequest{
					RequestId:   "asdasd-23234-fbzdfbv",
					ChainInfoId: 1,
					Address:     addr,
				},
			},
			want: &keypb.GetDidInfoResponse{
				Code: int32(code.ErrChainServiceUnReachable),
				Msg:  errNilChainClient,
			},
		},
		{
			name: code.ErrSendCallContactReq.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrSendCallContactReqSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetDidInfoRequest{
					RequestId:   "asdasd-23234-fbzdfbv",
					ChainInfoId: 1,
					Address:     addr,
				},
			},
			want: &keypb.GetDidInfoResponse{
				Code: int32(code.ErrSendCallContactReq),
				Msg:  code.ErrSendCallContactReq.String(),
			},
		},
		{
			name: code.ErrExecuteCallContact.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrExecuteCallContactSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetDidInfoRequest{
					RequestId:   "asdasd-23234-fbzdfbv",
					ChainInfoId: 1,
					Address:     addr,
				},
			},
			want: &keypb.GetDidInfoResponse{
				Code: int32(code.ErrExecuteCallContact),
				Msg:  code.ErrExecuteCallContact.String(),
			},
		},
		{
			name: code.ErrEmptyTxResponse.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrEmptyTxResponseSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetDidInfoRequest{
					RequestId:   "asdasd-23234-fbzdfbv",
					ChainInfoId: 1,
					Address:     addr,
				},
			},
			want: &keypb.GetDidInfoResponse{
				Code: int32(code.ErrEmptyTxResponse),
				Msg:  code.ErrEmptyTxResponse.String(),
			},
		},
		{
			name: code.ErrExecuteTx.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrExecuteTxSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetDidInfoRequest{
					RequestId:   "asdasd-23234-fbzdfbv",
					ChainInfoId: 1,
					Address:     addr,
				},
			},
			want: &keypb.GetDidInfoResponse{
				Code: int32(code.ErrExecuteTx),
				Msg:  code.ErrExecuteTx.String(),
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &GetDidInfoLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, _ := l.GetDidInfo(tt.args.in)

			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("GetDidInfo got = %v, want %v", got, tt.want)
			}
		})
	}
}
