package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/golang/mock/gomock"
	"github.com/zeromicro/go-zero/core/logx"
)

// 测试GetSkFromPkLogic
func TestGetSkFromPkLogic_GetSkFromPk(t *testing.T) {
	ctl := gomock.NewController(t)
	defer ctl.Finish()
	key := &types.SecretKey{
		SecretKeyID:       "202405212103141414",
		Scenario:          "SM2",
		PublicKey:         "048d10f957ee0b242ceeac0d8137600ae8ceebb702d0075a0a6ae7b00fa228e9b626a061255425098ac5c252c7c7d8742c6d00ee1bdc83f1592b38a8161202a924",
		PublicKeyFileName: "publicKeyFileName",
		PrivateKey:        "c757d51ed2b5487f2cc01ccbca5e8c71e0d3a00f7259742eb22e40552b4c1451",
		Status:            0,
	}
	mockDBAccessInterface := dbaccess.NewMockDBAccessInterface(ctl)
	mockDBAccessInterface.EXPECT().GetSecretKeyByPubKey(key.PublicKey).Return(key, nil)
	dbaccess.DBAccessService = mockDBAccessInterface
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		in *keypb.GetSkRequest
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.GetSkRequest{
					PubKey: key.PublicKey,
				},
			},
			want: &keypb.GetSkResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &GetSkFromPkLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.GetSkFromPk(tt.args.in)
			if (err != nil) != tt.wantErr {
				t.Errorf("GetSkFromPk() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试errorResponse
func TestGetSkFromPkLogic_errorResponse(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		code code.RespCode
		err  error
	}

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				code: code.ErrDidNotFindSk,
				err:  errors.New(code.ErrDidNotFindSk.String()),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &GetSkFromPkLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.errorResponse(tt.args.code, tt.args.err)
			if err == nil {
				t.Errorf("errorResponse() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试NewGetSkFromPkLogic
func TestNewGetSkFromPkLogic(t *testing.T) {
	type args struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
	}
	tests := []struct {
		name string
		args args
		want *CreateKeyLogic
	}{
		{
			name: "success",
			args: args{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := NewGetSkFromPkLogic(tt.args.ctx, tt.args.svcCtx); got == nil {
				t.Errorf("NewCreateKeyLogic() = %v, want %v", got, tt.want)
			}
		})
	}
}
