/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"encoding/base64"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/account"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/kms"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainweaver.org.cn/chainweaver/ida/common/v2/chain"
	"github.com/zeromicro/go-zero/core/logx"
)

// ImportKeyLogic defines the object to import key
type ImportKeyLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewImportKeyLogic create the object to import key
func NewImportKeyLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ImportKeyLogic {
	return &ImportKeyLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// ImportKey 导入私钥
func (l *ImportKeyLogic) ImportKey(in *keypb.ImportKeyRequest) (*keypb.ImportKeyResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId":    in.RequestId,
		"chainId":      in.ChainId,
		"subChainId":   in.SubChainId,
		"chainName":    in.ChainName,
		"keyMode":      in.Mode,
		"keyFileName":  in.KeyFileName,
		"keyFileType":  in.KeyFileType,
		"certFileName": in.CertFileName,
		"certFileType": in.CertFileType,
		"chainInfoId":  in.ChainInfoId,
		"alias":        in.Alias,
	}

	// 统计本地数据库中链账户数量
	count, err := l.svcCtx.KeyInfoModel.Count(in.ChainInfoId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrCountKeyInfo.String())
		return l.errorResponse(code.ErrCountKeyInfo, err), nil
	}

	// 只允许导入一条 key
	if count != 0 {
		fields["count"] = count
		fields["err"] = code.ErrNotSupportedMultiKeys.String()

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrNotSupportedMultiKeys.String())
		return l.errorResponse(code.ErrNotSupportedMultiKeys, nil), nil
	}

	// 加密私钥数据
	keyCipher, err := l.svcCtx.SymmetricKey.Encrypt(in.Key)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrSm4EncryptKey.String())
		return l.errorResponse(code.ErrSm4EncryptKey, err), nil
	}

	// 如果是证书模式，再加密证书数据
	certCipher := make([]byte, 0)
	if in.Mode == chain.KEY_MODE_CERT {
		certCipher, err = l.svcCtx.SymmetricKey.Encrypt(in.Cert)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			logFields := util.ConvertToLogFields(fields)
			l.Logger.WithFields(logFields...).Error(code.ErrSm4EncryptCert.String())
			return l.errorResponse(code.ErrSm4EncryptCert, err), nil
		}
	}

	accAddress, pubkey := "", ""

	// 如果不启用kms，则使用私钥生成地址
	if !l.svcCtx.Config.KmsConf.Enabled {

		// 根据私钥和配置的地址格式，生成对应的地址
		accAddress, err = account.GenAccountAddress(l.svcCtx.AddressType, in.Key)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrGenAccountAddress.String())
			return l.errorResponse(code.ErrGenAccountAddress, err), nil
		}
	} else {

		// 如果启用外接 kms，根据导入的 key 解析 kmsAdapterUrl 并创建 kmsClient
		var (
			kmsKey    kms.Key
			kmsClient = &kms.Client{
				Endpoint: l.svcCtx.Config.KmsConf.DefaultUrl,
			}
		)

		// 从 key 中解析出 kmsAdapterUrl 字段
		err = kmsKey.Unmarshal(in.Key)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrUnmarshalKmsKey.String())
			return l.errorResponse(code.ErrUnmarshalKmsKey, err), nil
		}

		// 使用 key 中解析出 kmsAdapterUrl
		if kmsKey.KmsAdapterUrl != "" {
			kmsClient.Endpoint = kmsKey.KmsAdapterUrl
		}

		// 如果启用kms，则使用kms生成公钥
		pubkey, err = kmsClient.GetPublicKey(string(in.Key))
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrGetPublicKey.String())
			return l.errorResponse(code.ErrGetPublicKey, err), nil
		}

		// 根据公钥生成对应地址
		accAddress, err = account.GenAccountAddressByPubkey(l.svcCtx.AddressType, pubkey)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrGenAccountAddress.String())
			return l.errorResponse(code.ErrGenAccountAddress, err), nil
		}

		// 缓存 kms client
		l.svcCtx.KmsClients.Store(in.ChainInfoId, kmsClient)
	}

	// 密文数据 base64 编码存储，避免特殊字符插入失败
	err = l.svcCtx.KeyInfoModel.Insert(in.ChainId, in.SubChainId, in.ChainName,
		in.Mode, base64.StdEncoding.EncodeToString(keyCipher), in.KeyFileName,
		in.KeyFileType, base64.StdEncoding.EncodeToString(certCipher), in.GetCertFileName(),
		in.GetCertFileType(), accAddress, in.Alias, pubkey, in.ChainInfoId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrInsertKeyInfo.String())
		return l.errorResponse(code.ErrInsertKeyInfo, err), nil
	}

	// 成功返回
	return l.successResponse(), nil
}

// errorResponse returns the error response.
func (l *ImportKeyLogic) errorResponse(code code.RespCode, err error) *keypb.ImportKeyResponse {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.ImportKeyResponse{
		Code: int32(code),
		Msg:  msg,
	}
}

// successResponse returns the success response.
func (l *ImportKeyLogic) successResponse() *keypb.ImportKeyResponse {
	return &keypb.ImportKeyResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
	}
}
