/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/common/v2/crypto/sym/sm4"
	"github.com/DATA-DOG/go-sqlmock"
	"github.com/zeromicro/go-zero/core/logx"
)

func createSuccessImportKeySVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(sqlmock.NewRows([]string{"count"}).AddRow(0))
	mock1.ExpectBegin()
	mock1.ExpectExec("INSERT INTO `key_info`").
		WithArgs(sqlmock.AnyArg(), sqlmock.AnyArg(), sqlmock.AnyArg(), "456", "b", "testChain456", "KEY_MODE_PK", sqlmock.AnyArg(),
			"", "", "", "", "", addr, "", "", "", 0, 0, "", "", "").
		WillReturnResult(sqlmock.NewResult(1, 1))
	mock1.ExpectCommit()
	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	return service
}

func createErrCountKeyInfoSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnError(errors.New(code.ErrCountKeyInfo.String()))

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	return service
}

func createMultiKeysSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(sqlmock.NewRows([]string{"count"}).AddRow(1))
	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func createAESFailedSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(sqlmock.NewRows([]string{"count"}).AddRow(0))
	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte("asdf"),
		},
	}
	return service
}

func createGenAccountFailedSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(sqlmock.NewRows([]string{"count"}).AddRow(0))
	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: "eos",
	}
	return service
}

func createInsertFailedSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(sqlmock.NewRows([]string{"count"}).AddRow(0))
	mock1.ExpectBegin()
	mock1.ExpectExec("INSERT INTO `key_info`").WillReturnError(errors.New(code.ErrInsertKeyInfo.String()))
	mock1.ExpectCommit()
	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	return service
}

func TestImportKeyLogic_ImportKey(t *testing.T) {
	type (
		fields struct {
			ctx    context.Context
			svcCtx *svc.ServiceContext
			Logger logx.Logger
		}

		args struct {
			in *keypb.ImportKeyRequest
		}
	)

	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.ImportKeyResponse
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createSuccessImportKeySVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.ImportKeyRequest{
					ChainId:    "456",
					SubChainId: "b",
					ChainName:  "testChain456",
					Mode:       "KEY_MODE_PK",
					Key:        keyBytes,
					RequestId:  "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.ImportKeyResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
		{
			name: code.ErrCountKeyInfo.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrCountKeyInfoSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.ImportKeyRequest{
					ChainId:    "456",
					SubChainId: "b",
					ChainName:  "testChain456",
					Mode:       "KEY_MODE_PK",
					Key:        keyBytes,
					RequestId:  "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.ImportKeyResponse{
				Code: int32(code.ErrCountKeyInfo),
				Msg:  code.ErrCountKeyInfo.String(),
			},
		},
		{
			name: code.ErrNotSupportedMultiKeys.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createMultiKeysSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.ImportKeyRequest{
					ChainId:    "456",
					SubChainId: "b",
					ChainName:  "testChain456",
					Mode:       "KEY_MODE_PK",
					Key:        keyBytes,
					RequestId:  "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.ImportKeyResponse{
				Code: int32(code.ErrNotSupportedMultiKeys),
				Msg:  code.ErrNotSupportedMultiKeys.String(),
			},
		},
		{
			name: code.ErrSm4EncryptKey.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createAESFailedSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.ImportKeyRequest{
					ChainId:    "456",
					SubChainId: "b",
					ChainName:  "testChain456",
					Mode:       "KEY_MODE_PK",
					Key:        keyBytes,
					RequestId:  "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.ImportKeyResponse{
				Code: int32(code.ErrSm4EncryptKey),
				Msg:  "SM4: invalid key size 4",
			},
		},
		{
			name: code.ErrGenAccountAddress.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createGenAccountFailedSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.ImportKeyRequest{
					ChainId:    "456",
					SubChainId: "b",
					ChainName:  "testChain456",
					Mode:       "KEY_MODE_PK",
					Key:        keyBytes,
					RequestId:  "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.ImportKeyResponse{
				Code: int32(code.ErrGenAccountAddress),
				Msg:  "unsupported address type: eos",
			},
		},
		{
			name: code.ErrInsertKeyInfo.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createInsertFailedSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.ImportKeyRequest{
					ChainId:    "456",
					SubChainId: "b",
					ChainName:  "testChain456",
					Mode:       "KEY_MODE_PK",
					Key:        keyBytes,
					RequestId:  "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.ImportKeyResponse{
				Code: int32(code.ErrInsertKeyInfo),
				Msg:  code.ErrInsertKeyInfo.String(),
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &ImportKeyLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, _ := l.ImportKey(tt.args.in)

			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("ImportKey got = %v, want %v", got, tt.want)
			}
		})
	}
}
