/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	commonpb "chainweaver.org.cn/chainweaver/ida/common/v2/pb/commonpb"
	"github.com/zeromicro/go-zero/core/logx"
)

// ListAccountLogic defines the object to list account
type ListAccountLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewListAccountLogic create the object to list account
func NewListAccountLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListAccountLogic {
	return &ListAccountLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// ListAccount 链账户列表
func (l *ListAccountLogic) ListAccount(in *keypb.ListAccountRequest) (*keypb.ListAccountResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId":   in.RequestId,
		"pageNum":     in.PageNumber,
		"pageSize":    in.PageSize,
		"chainInfoId": in.ChainInfoId,
	}

	// 校验 limit 参数
	limit, offset := 10, 0
	if in.PageSize > 0 {
		limit = int(in.PageSize)
	}

	// 校验 offset 参数
	if in.PageNumber > 0 {
		offset = int(in.PageSize * (in.PageNumber - 1))
	}

	// 从数据库中统计链账户数量
	total, err := l.svcCtx.KeyInfoModel.Count(int32(in.GetChainInfoId()))
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrCountKeyInfo.String())
		return &keypb.ListAccountResponse{
			Code: int32(code.ErrCountKeyInfo),
			Msg:  err.Error(),
		}, nil
	}

	// 查询链账户列表
	infos, authUsers, err := l.svcCtx.KeyInfoModel.List(limit, offset, in.GetChainInfoId())
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrListKeyInfo.String())
		return &keypb.ListAccountResponse{
			Code: int32(code.ErrListKeyInfo),
			Msg:  err.Error(),
		}, nil
	}

	// 成功返回
	return &keypb.ListAccountResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		Data: &keypb.ListAccountResponse_ListAccountData{

			// 组装分页信息
			Pagination: &commonpb.Pagination{
				PageNumber: in.PageNumber,
				PageSize:   in.PageSize,
				Total:      total,
			},

			// 转成 response data 数据结构
			List: util.ConvertToAccountData(infos, authUsers),
		},
	}, nil
}
