/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"encoding/base64"
	"encoding/pem"
	"errors"
	"fmt"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/csp/pkcs11"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/kms"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/common/v2/crypto"
	"chainmaker.org/chainmaker/common/v2/crypto/x509"
	"chainmaker.org/chainmaker/common/v2/json"
	"chainmaker.org/chainmaker/pb-go/v2/accesscontrol"
	"chainmaker.org/chainmaker/pb-go/v2/common"
	sdk "chainmaker.org/chainmaker/sdk-go/v2"
	chainservice "chainweaver.org.cn/chainweaver/ida/chain-service/v2/chain"
	"chainweaver.org.cn/chainweaver/ida/common/v2/chain"
	"github.com/zeromicro/go-zero/core/logx"
)

// SignLogic defines the object to sign
type SignLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewSignLogic create the object to sign
func NewSignLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SignLogic {
	return &SignLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// Sign 签名
func (l *SignLogic) Sign(in *keypb.SignRequest) (*keypb.SignResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId":   in.RequestId,
		"chainId":     in.GetChainId(),
		"subChainId":  in.GetSubChainId(),
		"userId":      in.GetUserId(),
		"chainInfoId": in.ChainInfoId,
	}

	// 根据 chainId 查询链账户信息
	ki, err := l.svcCtx.KeyInfoModel.QueryByChainInfoId(in.ChainInfoId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrQueryByChainInfoId.String())
		return l.errorResponse(code.ErrQueryByChainInfoId, err), nil
	}

	// gorm find 会过滤掉 RecordNotFound 错误，即没找到还是会返回一个非 nil 的对象，只不过里面的值都是默认零值
	if ki.ID == 0 {

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrNotFoundKey.String())
		return l.errorResponse(code.ErrNotFoundKey, nil), nil
	}

	// base64 解码私钥密文
	kc, err := base64.StdEncoding.DecodeString(ki.KeyCipher)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrBase64DecodeKeyCipher.String())
		return l.errorResponse(code.ErrBase64DecodeKeyCipher, err), nil
	}

	// 解密私钥数据
	k, err := l.svcCtx.SymmetricKey.Decrypt(kc)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrSm4DecryptKey.String())
		return l.errorResponse(code.ErrSm4DecryptKey, err), nil
	}

	// 如果是证书模式，再解密证书数据
	var (
		cert  *x509.Certificate
		orgId string
	)
	if ki.KeyMode == chain.KEY_MODE_CERT {
		var cc, certPem []byte

		// base64 解码证书密文
		cc, err = base64.StdEncoding.DecodeString(ki.CertCipher)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrBase64DecodeCertCipher.String())
			return l.errorResponse(code.ErrBase64DecodeCertCipher, err), nil
		}

		// 解密证书数据
		certPem, err = l.svcCtx.SymmetricKey.Decrypt(cc)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrSm4DecryptCert.String())
			return l.errorResponse(code.ErrSm4DecryptCert, err), nil
		}

		// 解析 pem 格式证书
		pemBlock, _ := pem.Decode(certPem)
		if pemBlock == nil {
			err = fmt.Errorf("%s", code.ErrDecodePem.String())
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrDecodePem.String())
			return nil, err
		}

		// 解析 x509 证书
		cert, err = x509.ParseCertificate(pemBlock.Bytes)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrParseCertFromPem.String())
			return nil, fmt.Errorf("%s,error: %v", code.ErrDecodePem.String(), err)
		}

		// 从证书中获取 orgId
		if len(cert.Issuer.Organization) > 0 {
			orgId = cert.Issuer.Organization[0]
		}
	}

	// json 反序列化得到 tx payload 对象
	var payload common.Payload
	err = json.Unmarshal(in.Payload, &payload)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrJsonUnmarshalPayload.String())
		return l.errorResponse(code.ErrJsonUnmarshalPayload, err), nil
	}

	// 签名
	signature, memInfo, err := l.signTxPayload(k, payload, ki.KeyMode, orgId, in.RequestId, ki.PublicKey,
		cert, fields, in.ChainInfoId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrSignTxPayload.String())
		return l.errorResponse(code.ErrSignTxPayload, err), nil
	}

	// 成功返回
	return l.successResponse(&keypb.SignResponseSignData{
		Signature: signature,

		// 转成 response data 数据结构
		Signer: util.ConvertToSignResponseMember(memInfo),
	}), nil
}

func (l *SignLogic) signTxPayload(sk []byte, payload common.Payload, keyMode, orgId, requestId, pkPem string,
	cert *x509.Certificate, fields map[string]interface{},
	chainInfoId uint32) (signature []byte, memInfo *accesscontrol.Member, err error) {

	// 使用外接 kms 签名
	if l.svcCtx.Config.KmsConf.Enabled {

		// 查询缓存中的 kmsClient
		kmsClient, ok := l.svcCtx.KmsClients.Load(int32(chainInfoId))
		if !ok || kmsClient == nil {

			// 之前没有缓存过 kmsClient，需要重新创建
			newClient := &kms.Client{
				Endpoint: l.svcCtx.Config.KmsConf.DefaultUrl,
			}

			// 如果 keyJson 中有指定 KmsAdapterUrl，则替换使用
			var kmsKey kms.Key
			err = kmsKey.Unmarshal(sk)
			if err != nil {
				return nil, nil, fmt.Errorf("unmarshal kms key failed: %v", err)
			}

			// 缓存新的 kmsClient
			newClient.Endpoint = kmsKey.KmsAdapterUrl
			kmsClient = newClient
			l.svcCtx.KmsClients.Store(int32(chainInfoId), newClient)
		}

		// kms 签名 tx payload
		var signatureStr string
		signatureStr, err = kmsClient.(*kms.Client).SignTxPayload(string(sk), &payload)
		if err != nil {
			return nil, nil, fmt.Errorf("kms SignTxPayload failed: %v", err)
		}

		signature = []byte(signatureStr)

		// 创建 member info
		memInfo, err = kmsClient.(*kms.Client).CreateSignerMember(keyMode, orgId, pkPem, cert)
		if err != nil {
			return nil, nil, fmt.Errorf("kms CreateSignerMember failed: %v", err)
		}

		// 使用本地私钥签名
	} else {

		// 将私钥数据转成签名用的 PrivateKey 对象
		var (
			privKey crypto.PrivateKey
			signer  sdk.Signer
		)
		privKey, err = pkcs11.GetSignKey(l.svcCtx.Config.Pkcs11Conf, sk)
		if err != nil {
			return nil, nil, fmt.Errorf("GetSignKey failed: %v", err)
		}

		// 创建签名对象 signer
		signer, err = l.createSigner(fields, keyMode, privKey, cert, chainInfoId, requestId, orgId)
		if err != nil {
			return nil, nil, fmt.Errorf("createSigner failed: %v", err)
		}

		// 对 tx payload 签名
		signature, err = signer.Sign(&payload)
		if err != nil {
			return nil, nil, fmt.Errorf("signer.Sign failed: %v", err)
		}

		// 获取签名者成员信息
		memInfo, err = signer.NewMember()
		if err != nil {
			return nil, nil, fmt.Errorf("signer.NewMember failed: %v", err)
		}
	}

	return
}

// 组装签名结构
func (l *SignLogic) createSigner(fields map[string]interface{}, keyMode string,
	privKey crypto.PrivateKey, cert *x509.Certificate, chainInfoId uint32, requestId, orgId string) (sdk.Signer, error) {
	var signer sdk.Signer
	switch keyMode {

	//  创建证书模式的签名对象
	case chain.KEY_MODE_CERT:
		fields["keyMode"] = chain.KEY_MODE_CERT

		// 初始化证书模式签名对象
		signer = &sdk.CertModeSigner{
			PrivateKey: privKey,
			Cert:       cert,
			OrgId:      orgId,
		}
		fields["orgId"] = orgId

	// 创建公钥模式的签名对象
	case chain.KEY_MODE_PK:
		fields["keyMode"] = chain.KEY_MODE_PK
		hashType, err := l.getSignHashType(int32(chainInfoId), requestId)
		if err != nil {
			fields["err"] = err

			// 转成结构化日志信息
			l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error("failed to getSignHashType")
			return nil, err
		}

		fields["hashType"] = hashType

		// 初始化公钥模式签名对象
		signer = &sdk.PublicModeSigner{
			PrivateKey: privKey,
			HashType:   hashType,
		}

	// 链账户模式未知，报错返回
	default:
		fields["keyMode"] = keyMode

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrUnknownKeyMode.String())
		return nil, fmt.Errorf("%s", code.ErrUnknownKeyMode.String())
	}

	return signer, nil
}

// errorResponse returns the error response.
func (l *SignLogic) errorResponse(code code.RespCode, err error) *keypb.SignResponse {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.SignResponse{
		Code: int32(code),
		Msg:  msg,
	}
}

// successResponse returns the success response.
func (l *SignLogic) successResponse(data *keypb.SignResponseSignData) *keypb.SignResponse {
	return &keypb.SignResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		Data: data,
	}
}

func (l *SignLogic) getSignHashType(chainInfoId int32, requestId string) (crypto.HashType, error) {

	// 检查 chain-service 客户端是否初始化
	if l.svcCtx.ChainClient == nil {
		return 0, errors.New(errNilChainClient)
	}

	// 组装 GetChainInfo grpc 请求结构
	in := &chainservice.GetChainInfoRequest{
		Id:        chainInfoId,
		RequestId: requestId,
	}

	// 发送 GetChainInfo grpc 请求
	res, err := l.svcCtx.ChainClient.GetChainInfo(l.ctx, in)
	if err != nil {
		return 0, fmt.Errorf("failed to send GetChainInfo req,err: %v", err)
	}

	// 检查 grpc 返回结果
	if res.Code != int32(code.Success) {
		return 0, fmt.Errorf("failed to execute GetChainInfo,err: %v", res.Msg)
	}

	// 检查区块链配置是否完成
	if res.Data == nil || res.Data.HashType == "" {
		return 0, fmt.Errorf("empty chain config(%v) or no hash type(%v)", res.Data == nil, res.Data.HashType == "")
	}

	// 根据hash类型，转成签名对应的常量值
	switch res.Data.HashType {
	case chain.HASH_TYPE_SM3:
		return crypto.HASH_TYPE_SM3, nil
	case chain.HASH_TYPE_SHA256:
		return crypto.HASH_TYPE_SHA256, nil
	case chain.HASH_TYPE_SHA3_256:
		return crypto.HASH_TYPE_SHA3_256, nil
	default:
		return 0, fmt.Errorf("unknown hash type: %v", res.Data.HashType)
	}
}
