/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"
	"errors"
	"testing"
	"time"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/common/v2/crypto/sym/sm4"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/chain"
	chainMock "chainweaver.org.cn/chainweaver/ida/chain-service/v2/mock"
	"chainweaver.org.cn/chainweaver/ida/chain-service/v2/pb/chainpb"
	"github.com/DATA-DOG/go-sqlmock"
	"github.com/zeromicro/go-zero/core/logx"
	"go.uber.org/mock/gomock"
)

func createSuccessSignSVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	ctrl := gomock.NewController(t)
	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "key_mode", "key_cipher", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", "KEY_MODE_PK", keyCipher, time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)

	chainClient := chainMock.NewMockChain(ctrl)
	chainResp := &chain.GetChainInfoResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		Data: &chainpb.ChainInfoData{
			HashType: "SM3",
		},
	}
	chainClient.EXPECT().GetChainInfo(context.Background(), gomock.Any()).AnyTimes().Return(chainResp, nil)

	service := &svc.ServiceContext{
		ChainClient:  chainClient,
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
	}
	return service
}

func createFailedQueryByChainIdAndUserIdSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnError(errors.New(code.ErrQueryByChainInfoId.String()))

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func createNoAuthorizedSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(sqlmock.NewRows(nil))

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func createErrBase64DecodeKeyCipherSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "key_mode", "key_cipher", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", "KEY_MODE_PK", "asdfsdf", time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
	}
	return service
}

func createErrSm4DecryptKeySVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "key_mode", "key_cipher", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", "KEY_MODE_PK", keyCipher, time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte("asdd"),
		},
	}
	return service
}

func createErrSignTxPayloadSVC() *svc.ServiceContext {
	keyModel1, mock1 := newTestDB()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "key_mode", "key_cipher", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", "KEY_MODE_PK2", keyCipher, time.Now(), time.Now())
	mock1.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)

	service := &svc.ServiceContext{
		KeyInfoModel: keyModel1,
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
	}
	return service
}

func TestSignLogic_Sign(t *testing.T) {
	type (
		fields struct {
			ctx    context.Context
			svcCtx *svc.ServiceContext
			Logger logx.Logger
		}

		args struct {
			in *keypb.SignRequest
		}
	)

	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.SignResponse
	}{
		{
			name: code.Success.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createSuccessSignSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.SignRequest{
					ChainInfoId: 1,
					Payload:     []byte("{\"chain_id\":\"chain1\"}"),
					RequestId:   "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.SignResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
		{
			name: code.ErrQueryByChainInfoId.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createFailedQueryByChainIdAndUserIdSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.SignRequest{
					ChainInfoId: 1,
					Payload:     []byte("{\"chain_id\":\"chain1\"}"),
					RequestId:   "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.SignResponse{
				Code: int32(code.ErrQueryByChainInfoId),
				Msg:  code.ErrQueryByChainInfoId.String(),
			},
		},
		{
			name: code.ErrNotFoundKey.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createNoAuthorizedSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.SignRequest{
					ChainInfoId: 1,
					Payload:     []byte("{\"chain_id\":\"chain1\"}"),
					RequestId:   "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.SignResponse{
				Code: int32(code.ErrNotFoundKey),
				Msg:  code.ErrNotFoundKey.String(),
			},
		},
		{
			name: code.ErrBase64DecodeKeyCipher.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrBase64DecodeKeyCipherSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.SignRequest{
					ChainInfoId: 1,
					Payload:     []byte("{\"chain_id\":\"chain1\"}"),
					RequestId:   "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.SignResponse{
				Code: int32(code.ErrBase64DecodeKeyCipher),
				Msg:  "illegal base64 data at input byte 4",
			},
		},
		{
			name: code.ErrSm4DecryptKey.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrSm4DecryptKeySVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.SignRequest{
					ChainInfoId: 1,
					Payload:     []byte("{\"chain_id\":\"chain1\"}"),
					RequestId:   "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.SignResponse{
				Code: int32(code.ErrSm4DecryptKey),
				Msg:  "SM4: invalid key size 4",
			},
		},
		{
			name: code.ErrSignTxPayload.String(),
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createErrSignTxPayloadSVC(),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.SignRequest{
					ChainInfoId: 1,
					Payload:     []byte("{\"chain_id\":\"chain1\"}"),
					RequestId:   "asdasd-23234-fbzdfbv",
				},
			},
			want: &keypb.SignResponse{
				Code: int32(code.ErrSignTxPayload),
				Msg:  "createSigner failed: unknown key mode",
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &SignLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, _ := l.Sign(tt.args.in)

			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("Sign got = %v, want %v", got, tt.want)
			}
		})
	}
}
