/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package model defines db operation
package model

import (
	"context"
	"errors"
	"strings"
	"time"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/config"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/model/table"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"

	"chainweaver.org.cn/chainweaver/ida/common/v2/db"
	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/driver/mysql"
	"gorm.io/driver/postgres"
	"gorm.io/gorm"
)

const (

	// MaxLifetime 最大连接时间，防止使用无效的连接，单位：s
	MaxLifetime = 500

	// MaxIdleConns 空闲连接池中连接的最大数量
	MaxIdleConns = 10

	// MaxOpenConns 数据库连接的最大数量
	MaxOpenConns = 10

	// DefaultStringSize string 类型字段的默认长度
	DefaultStringSize = 256

	// DisableDatetimePrecision 禁用 datetime 精度，MySQL 5.6 之前的数据库不支持
	DisableDatetimePrecision = true

	// DontSupportRenameIndex 重命名索引时采用删除并新建的方式，MySQL 5.7 之前的数据库和 MariaDB 不支持重命名索引
	DontSupportRenameIndex = true

	// DontSupportRenameColumn 用 `change` 重命名列，MySQL 8 之前的数据库和 MariaDB 不支持重命名列
	DontSupportRenameColumn = true

	// SkipInitializeWithVersion 根据当前 MySQL 版本自动配置
	SkipInitializeWithVersion = false
)

// KeyInfoModel defines the object to operate db
type KeyInfoModel struct {
	DB *gorm.DB
}

// NewKeyInfoModel create the object to operate db
func NewKeyInfoModel(c config.DBConf) *KeyInfoModel {
	fields := map[string]interface{}{
		"Type": c.Type,
		"DSN":  c.DNS,
	}

	var (
		gormDB *gorm.DB
		err    error
	)
	ctx := context.Background()
	switch c.Type {
	case db.DBTypeMysql, db.DBTypeKingBaseMysql:
		gormDB, err = gorm.Open(mysql.New(mysql.Config{
			DSN:                       c.DNS,
			SkipInitializeWithVersion: SkipInitializeWithVersion,
			DefaultStringSize:         DefaultStringSize,
			DisableDatetimePrecision:  DisableDatetimePrecision,
			DontSupportRenameIndex:    DontSupportRenameIndex,
			DontSupportRenameColumn:   DontSupportRenameColumn,
		}), &gorm.Config{})
	case db.DBTypeKingBasePgsql, db.DBTypePostgres:
		gormDB, err = gorm.Open(postgres.Open(c.DNS), &gorm.Config{})
	default:
		logx.WithContext(ctx).WithFields(util.ConvertToLogFields(fields)...).Error("unknown gormDB type")
		panic("unknown gormDB type")
	}

	if err != nil {
		fields["error"] = err
		logx.WithContext(ctx).WithFields(util.ConvertToLogFields(fields)...).Error("failed to open gormDB connection")
		panic(err)
	}

	sqlDb, err := gormDB.DB()
	if err != nil {
		fields["error"] = err
		logx.WithContext(ctx).WithFields(util.ConvertToLogFields(fields)...).Error("failed to get gormDB connection")
		panic(err)
	}

	sqlDb.SetMaxIdleConns(MaxIdleConns)
	sqlDb.SetMaxOpenConns(MaxOpenConns)
	sqlDb.SetConnMaxLifetime(MaxLifetime * time.Second)

	if err = sqlDb.Ping(); err != nil {
		fields["error"] = err
		logx.WithContext(ctx).WithFields(util.ConvertToLogFields(fields)...).Error("failed to ping gormDB connection")
		panic(err)
	}

	logx.WithContext(ctx).WithFields(util.ConvertToLogFields(fields)...).Info("success to connect gormDB")

	// 自动建表
	err = gormDB.AutoMigrate(&table.KeyInfo{})
	if err != nil {
		fields["error"] = err
		logx.WithContext(ctx).WithFields(util.ConvertToLogFields(fields)...).Error("failed to auto migrate tables")
		panic(err)
	}

	logx.WithContext(ctx).WithFields(util.ConvertToLogFields(fields)...).Info("success to auto migrate tables")

	return &KeyInfoModel{
		DB: gormDB,
	}
}

// Insert 插入链账户
func (km *KeyInfoModel) Insert(chainId, subChainId, chainName, keyMode, keyCipher, keyFileName, keyFileType,
	certCipher, certFileName, certFileType, accAddress, alias, pubKey string, chainInfoId int32) error {
	tx := km.DB.Begin()
	var err error
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	info := table.KeyInfo{
		ChainId:        chainId,
		SubChainId:     subChainId,
		ChainName:      chainName,
		KeyMode:        keyMode,
		KeyCipher:      keyCipher,
		KeyFileName:    keyFileName,
		KeyFileType:    keyFileType,
		CertCipher:     certCipher,
		CertFileName:   certFileName,
		CertFileType:   certFileType,
		AccountAddress: accAddress,
		ChainInfoId:    uint(chainInfoId),
		Alias:          alias,
		PublicKey:      pubKey,
	}
	err = tx.Save(&info).Error
	if err != nil {
		return err
	}

	return nil
}

// QueryByChainIdAndUserId 根据 chain id、sub chain id 查询链账户
func (km *KeyInfoModel) QueryByChainIdAndUserId(chainId, subChainId string,
	userId uint32) (ki *table.KeyInfo, err error) {
	if userId != 0 {
		err = km.DB.Joins("JOIN auth_user ON auth_user.key_id = key_info.id AND key_info.chain_id = ? "+
			"AND key_info.sub_chain_id = ? AND auth_user.user_id = ?", chainId, subChainId, userId).Find(&ki).Error
		if err != nil {
			return nil, err
		}
	} else {

		// 刚导入 key 时还没有授权用户，这时只能根据 chain_id、sub_chain_id 查询key
		err = km.DB.Where("chain_id = ? and sub_chain_id = ?", chainId, subChainId).Find(&ki).Error
		if err != nil {
			return nil, err
		}
	}

	return ki, nil
}

// QueryByChainInfoId 查询指定链的账户，目前是一条链一个账户
func (km *KeyInfoModel) QueryByChainInfoId(chainInfoId uint32) (ki *table.KeyInfo, err error) {
	err = km.DB.Where("chain_info_id = ?", chainInfoId).Find(&ki).Error
	if err != nil {
		return nil, err
	}

	return ki, nil
}

// Count 链账户数量
func (km *KeyInfoModel) Count(chainInfoId int32) (count int64, err error) {
	tx := km.DB.Model(&table.KeyInfo{})
	if chainInfoId != 0 {
		tx = tx.Where("chain_info_id = ?", chainInfoId)
	}

	err = tx.Count(&count).Error
	if err != nil {
		return 0, err
	}

	return count, nil
}

// List 链账户列表
func (km *KeyInfoModel) List(limit, offset int, chainInfoId int32) (
	infos []table.KeyInfo, authUsers []map[uint32]string, err error) {
	tx := km.DB.Model(&table.KeyInfo{})
	if chainInfoId != 0 {
		tx = tx.Where("chain_info_id = ?", chainInfoId)
	}

	err = tx.Limit(limit).Offset(offset).Find(&infos).Error
	if err != nil {
		return nil, nil, err
	}

	//authUsers := make([]map[uint32]string, len(infos))
	//for i, k := range infos {
	//	var aus []table.AuthUser
	//	err := km.DB.Where("key_id = ?", k.Id).Find(&aus).Error
	//	if err != nil {
	//
	//		// 可能当前 key 还没有授权用户
	//		if errors.Is(err, gorm.ErrRecordNotFound) {
	//			continue
	//		}
	//
	//		return nil, nil, err
	//	}
	//
	//	authUsers[i] = make(map[uint32]string)
	//	for _, au := range aus {
	//		authUsers[i][au.UserId] = au.UserName
	//	}
	//}

	return infos, nil, nil
}

// QueryAuthUserId 根据 key id 和 user id 查询授权表记录 id
func (km *KeyInfoModel) QueryAuthUserId(keyId, userId uint32) (uint, error) {
	var au table.AuthUser
	err := km.DB.Where("key_id = ? and user_id = ?", keyId, userId).Find(&au).Error
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return 0, nil
		}

		return 0, err
	}

	return au.Id, nil
}

// DeleteAuthUsers 删除链账户授权用户
func (km *KeyInfoModel) DeleteAuthUsers(keyId uint, needDeleteAuthUsers []uint32) error {
	for _, uid := range needDeleteAuthUsers {
		err := km.DB.Where("key_id = ? and user_id = ?", keyId, uid).Delete(&table.AuthUser{}).Error
		if err != nil {
			return err
		}
	}

	return nil
}

// Update 更新链账户信息
func (km *KeyInfoModel) Update(keyId uint, accName, socialCreditCode string,
	authUsers map[uint32]string, accountTypes []string, did string) error {
	tx := km.DB.Begin()
	var err error
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	err = tx.Model(&table.KeyInfo{}).Where("id = ?", keyId).
		Updates(map[string]interface{}{
			"account_name":               accName,
			"account_social_credit_code": socialCreditCode,

			// 把数组 accountTypes 用 & 拼接成一个字符串
			"account_type": strings.Join(accountTypes, "&"),
			//"register_id":  registerId,
			"did": did,
		}).Error
	if err != nil {
		return err
	}

	//// 查询旧的 auth_user
	//_, oldAuthUsers, err := km.QueryByKeyId(uint32(keyId))
	//if err != nil {
	//	return err
	//}
	//
	//// 过滤出需要移除的 auth_user
	//needDeleteAuthUsers := make([]uint32, 0)
	//for oid := range oldAuthUsers {
	//	var ok bool
	//	for nid := range authUsers {
	//		if nid == oid {
	//			ok = true
	//			break
	//		}
	//	}
	//
	//	if !ok {
	//		needDeleteAuthUsers = append(needDeleteAuthUsers, oid)
	//	}
	//}
	//
	//// 删除旧的 auth_user
	//if len(needDeleteAuthUsers) > 0 {
	//	err = km.DeleteAuthUsers(keyId, needDeleteAuthUsers)
	//	if err != nil {
	//		return err
	//	}
	//}
	//
	//// 更新最新的 auth_user
	//for userId, userName := range authUsers {
	//	id, err := km.QueryAuthUserId(uint32(keyId), userId)
	//	if err != nil {
	//		return err
	//	}
	//
	//	// 已经存在 auth_user，更新即可，避免重复插入
	//	if id == 0 {
	//		au := &table.AuthUser{
	//			Id:       id,
	//			KeyId:    keyId,
	//			UserId:   userId,
	//			UserName: userName,
	//		}
	//		err = tx.Create(au).Error
	//		if err != nil {
	//			return err
	//		}
	//	} else {
	//		err = tx.Model(&table.AuthUser{}).Where("id = ?", id).
	//			Updates(map[string]interface{}{
	//				"key_id":    keyId,
	//				"user_id":   userId,
	//				"user_name": userName,
	//			}).Error
	//		if err != nil {
	//			return err
	//		}
	//	}
	//
	//}

	return nil
}

// DeleteByKeyId 根据 key id 删除链账户
func (km *KeyInfoModel) DeleteByKeyId(keyId uint32) (err error) {
	tx := km.DB.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	//// 先删除授权用户
	//err = tx.Where("key_id = ?", keyId).Delete(&table.AuthUser{}).Error
	//if err != nil {
	//	return err
	//}

	// 再删除私钥
	err = tx.Where("id = ?", keyId).Delete(&table.KeyInfo{}).Error
	if err != nil {
		return err
	}

	return nil
}

// QueryByKeyId 根据 key id 查询链账户
func (km *KeyInfoModel) QueryByKeyId(keyId uint32) (ki *table.KeyInfo, aus map[uint32]string, err error) {
	err = km.DB.Where("id = ?", keyId).Find(&ki).Error
	if err != nil {
		return nil, nil, err
	}

	//var aus []table.AuthUser
	//err = km.DB.Where("key_id = ?", keyId).Find(&aus).Error
	//if err != nil {
	//	if errors.Is(err, gorm.ErrRecordNotFound) {
	//		return &ki, nil, nil
	//	}
	//
	//	return nil, nil, err
	//}
	//
	//authUsers := make(map[uint32]string)
	//for _, au := range aus {
	//	authUsers[au.UserId] = au.UserName
	//}

	return
}
