package svc

import (
	"crypto/rand"
	"encoding/hex"
	"errors"
	"fmt"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	mira_config "chainweaver.org.cn/chainweaver/mira/mira-common/config"
	"chainweaver.org.cn/chainweaver/mira/mira-common/crypto"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/tjfoc/gmsm/sm2"
	"github.com/tjfoc/gmsm/sm4"
	"github.com/tjfoc/gmsm/x509"
)

// SM2KeyPairCreate 生成SM2密钥对
func SM2KeyPairCreate() (*types.SecretKey, error) {
	// 生成私钥
	privateKey, err := sm2.GenerateKey(rand.Reader)
	if err != nil {
		return nil, err
	}
	// 根据私钥获取公钥
	publicKey := &privateKey.PublicKey
	//转换成hex
	privateKeyHex := x509.WritePrivateKeyToHex(privateKey)
	publicKeyHex := x509.WritePublicKeyToHex(publicKey)
	//构造结构体
	key := types.SecretKey{
		Scenario:   keypb.AlgoType_name[1],
		PublicKey:  publicKeyHex,
		PrivateKey: privateKeyHex,
	}
	return &key, nil
}

// SM2Encrypt SM2加密
func SM2Encrypt(pubKeyHex string, plainText string) (string, error) {
	// 判断是否是有效的hex
	if !util.IsAvailablePk(pubKeyHex) {
		return "", errors.New("PK encoding type is not hex")
	}
	// 解析公钥
	pubKey, err := x509.ReadPublicKeyFromHex(pubKeyHex)
	if err != nil {
		return "", fmt.Errorf("error parsing public key: %v", err)
	}
	// 使用公钥进行加密, SM2加解密有两种模式：C1C2C3和C1C3C2，这里使用C1C2C3（国密标准推荐使用C1C2C3）
	cipherText, err := sm2.Encrypt(pubKey, []byte(plainText), rand.Reader, sm2.C1C2C3)
	if err != nil {
		return "", fmt.Errorf("error encrypting with SM2: %v", err)
	}
	// 构造结果为hex
	result := hex.EncodeToString(cipherText)
	return result, nil
}

// SM2Decrypt SM2解密
func SM2Decrypt(privateKey, cipherText string) (string, error) {
	// 转换格式
	cipherBytes, err := hex.DecodeString(cipherText)
	if err != nil {
		return "", fmt.Errorf("error decoding cipher text: %v", err)
	}
	// 解析私钥
	priv, err := x509.ReadPrivateKeyFromHex(privateKey)
	if err != nil {
		return "", fmt.Errorf("error parsing private key: %v", err)
	}
	// 使用私钥进行加密, SM2有两种模式：C1C2C3和C1C3C2，这里使用C1C2C3（国密标准推荐使用C1C2C3）
	plaintext, err := sm2.Decrypt(priv, cipherBytes, sm2.C1C2C3)
	if err != nil {
		return "", fmt.Errorf("error decrypting with SM2: %v", err)
	}
	return string(plaintext), nil
}

// EncWithDeK 数字信封加密
func EncWithDeK(pubKeyHex, plainText string) (string, string, error) {
	// 初始化加密模式，硬件或者软件
	csp, err := crypto.InitCrypto(&mira_config.CryptoConfig{
		Soft: true,
	})
	if err != nil {
		return "", "", err
	}
	// 解析hex类型的公钥
	pubKey, err := x509.ReadPublicKeyFromHex(pubKeyHex)
	if err != nil {
		return "", "", err
	}
	// 类型转换
	pubKeyByte, err := x509.MarshalSm2PublicKey(pubKey)
	if err != nil {
		return "", "", err
	}
	// 数字信封加密
	cipherText, kek, err := csp.EncWithDek("SM2SM4DataEnvelope", string(pubKeyByte), []byte(plainText))
	if err != nil {
		return "", "", err
	}
	// 构造结果为hex
	cipherTextResult := hex.EncodeToString(cipherText)
	kekResult := hex.EncodeToString(kek)
	return cipherTextResult, kekResult, nil
}

// EncWithDekList 数字信封加密List
func EncWithDekList(pubKeyHexList []string, plainText string) (string, []string, error) {
	// 初始化加密模式，硬件或者软件
	csp, err := crypto.InitCrypto(&mira_config.CryptoConfig{
		Soft: true,
	})
	if err != nil {
		return "", nil, err
	}
	// 类型转换
	pubKeyStringList, err := util.ConvertPubKeyHexListToByteList(pubKeyHexList)
	if err != nil {
		return "", nil, err
	}
	// 根据公钥列表加密
	cipherText, encDekList, err := csp.EncWithDekList("SM2SM4DataEnvelope", pubKeyStringList, []byte(plainText))
	if err != nil {
		return "", nil, err
	}
	// 构造结果
	cipherTextResult := hex.EncodeToString(cipherText)
	kekList := util.ConvertBytesList2HexList(encDekList)
	return cipherTextResult, kekList, nil
}

// DecByKeK 数字信封解密
func DecByKeK(privateKey, cipherText, kek string) (string, error) {
	// 初始化加密模式，硬件或者软件
	csp, err := crypto.InitCrypto(&mira_config.CryptoConfig{
		Soft: true,
	})
	if err != nil {
		return "", err
	}
	// 类型转换
	privateKeyStruct, err := x509.ReadPrivateKeyFromHex(privateKey)
	if err != nil {
		return "", err
	}
	pubKeyByte, err := x509.MarshalSm2PrivateKey(privateKeyStruct, nil)
	if err != nil {
		return "", err
	}
	cipherBytes, _ := hex.DecodeString(cipherText)
	kekBytes, _ := hex.DecodeString(kek)
	// 数字信封解密
	plainText, err := csp.DecByKeK("SM2SM4DataEnvelope", string(pubKeyByte), cipherBytes, kekBytes)
	if err != nil {
		return "", err
	}
	return string(plainText), nil
}

// GenSymKey 生成对称密钥
func GenSymKey(algo keypb.AlgoType, keySize int32) (*types.SecretKey, error) {
	// 初始化加密模式，硬件或者软件
	csp, err := crypto.InitCrypto(&mira_config.CryptoConfig{
		Soft: true,
	})
	if err != nil {
		return nil, err
	}
	var key string
	var keyBytes []byte
	var algoType string
	// 判断算法类型
	switch algo {
	// 如果是SM4
	case keypb.AlgoType_SM4:
		algoType = "SM4"
		keyBytes, err = csp.GenSymKey("SM4", int(keySize))
		if err != nil {
			return nil, err
		}
		key = hex.EncodeToString(keyBytes)
		if len(key) > 16 {
			key = key[:16]
		}
	//	如果是AES
	case keypb.AlgoType_AES:
		algoType = "AES"
		keyBytes, err = csp.GenSymKey("AES", int(keySize))
		if err != nil {
			return nil, err
		}
		key = hex.EncodeToString(keyBytes)
		if len(key) > int(keySize) {
			key = key[:int(keySize)]
		}
	}
	secretKey := types.SecretKey{
		Scenario:   algoType,
		PublicKey:  key,
		PrivateKey: key,
	}
	return &secretKey, nil
}

// SM4Encrypt SM4加密
func SM4Encrypt(key string, plainText string) (string, error) {
	// 加密
	cipherBytes, err := sm4.Sm4Cbc([]byte(key), []byte(plainText), true)
	if err != nil {
		return "", err
	}
	// 构造结果
	result := hex.EncodeToString(cipherBytes)
	return result, nil
}

// SM4Decrypt SM4解密
func SM4Decrypt(key string, cipherText string) (string, error) {
	// 处理参数
	cipherBytes, err := hex.DecodeString(cipherText)
	if err != nil {
		return "", err
	}
	// 解密明文
	plainText, err := sm4.Sm4Cbc([]byte(key), cipherBytes, false)
	if err != nil {
		return "", err
	}
	return string(plainText), nil
}
