/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package svc defines service context
package svc

import (
	"encoding/base64"
	"sync"
	"time"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/config"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/kms"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/model"

	"chainmaker.org/chainmaker/common/v2/crypto"
	"chainmaker.org/chainmaker/common/v2/crypto/sym/aes"
	"chainmaker.org/chainmaker/common/v2/crypto/sym/sm4"
	chainservice "chainweaver.org.cn/chainweaver/ida/chain-service/v2/chain"
	"chainweaver.org.cn/chainweaver/ida/common/v2/grpc"
	"github.com/zeromicro/go-zero/core/logx"
)

// ServiceContext contain all context items for service running
type ServiceContext struct {
	Config       config.Config
	KeyInfoModel *model.KeyInfoModel

	// SymmetricKey 对称加解密对象，用于对私钥、证书数据加解密
	SymmetricKey crypto.SymmetricKey

	// 链上地址格式类型
	AddressType string

	// chain-service 客户端对象，用于发送交易上链
	ChainClient chainservice.Chain

	// 与链交互的 kms adapter 客户端,chainInfoId -> kms.Client
	KmsClients sync.Map
}

// NewServiceContext initiate all context items for service running
func NewServiceContext(c config.Config) *ServiceContext {
	svc := &ServiceContext{
		Config:       c,
		KeyInfoModel: model.NewKeyInfoModel(c.DBConf),
		AddressType:  c.AddressType,
	}

	// 初始化对称加解密对象
	switch c.SymmetricEncryptConf.Algo {
	case config.AlgoAes:
		svc.SymmetricKey = &aes.AESKey{
			Key: []byte(c.SymmetricEncryptConf.Key),
		}
	case config.AlgoSm4:
		svc.SymmetricKey = &sm4.SM4Key{
			Key: []byte(c.SymmetricEncryptConf.Key),
		}
	}

	// 初始化 chain-service 客户端
	svc.newChainClient(c.ChainServiceConf)

	// 初始化 KmsClients
	svc.intiKmsClients()
	return svc
}

// 初始化 chain-service 客户端
func (svc *ServiceContext) newChainClient(c config.ChainServiceConf) {
	go func() {
		for {

			// 创建 chain-service grpc client
			chainGrpcClient, err := grpc.CreateGRPCClient(c.CaCertFile, c.ClientCertFile, c.ClientKeyFile, c.DNS, c.Endpoint)
			if err != nil {
				logx.Errorf("failed to create grpc client for chain-service,err: %v", err)
				time.Sleep(time.Second * 3)
				continue
			}

			// 创建 chain-service 客户端
			svc.ChainClient = chainservice.NewChain(*chainGrpcClient)
			logx.Info("success to create chain-service client")
			return
		}
	}()
}

// 初始化 KmsClients
func (svc *ServiceContext) intiKmsClients() {
	go func() {

		// 如果不开启外接 kms，则直接返回
		if !svc.Config.KmsConf.Enabled {
			return
		}

		var offset = 0

		for {

			// 如果开启外接 kms，则解析本地 db 中已导入 keyJson 的 kmsAdapterUrl
			keyInfos, _, err := svc.KeyInfoModel.List(20, offset, 0)
			if err != nil {
				logx.Errorf("failed to list key infos,err: %v", err)
				time.Sleep(time.Second * 3)
				continue
			}

			for _, ki := range keyInfos {

				// base64 解码私钥密文
				kc, err2 := base64.StdEncoding.DecodeString(ki.KeyCipher)
				if err2 != nil {
					logx.Errorf("failed to base64 decode keyCipher,err: %v", err2)
					time.Sleep(time.Second * 3)
					continue
				}

				// 解密私钥数据
				kRaw, err2 := svc.SymmetricKey.Decrypt(kc)
				if err2 != nil {
					logx.Errorf("failed to SymmetricKey.Decrypt keyCipher,err: %v", err2)
					time.Sleep(time.Second * 3)
					continue
				}

				// 解析 KmsAdapterUrl
				var kmsKey kms.Key
				err2 = kmsKey.Unmarshal(kRaw)
				if err2 != nil {
					logx.Errorf("failed to kmsKey.Unmarshal,err: %v", err2)
					time.Sleep(time.Second * 3)
					continue
				}

				// 缓存 kmsClient
				kmsClient := &kms.Client{
					Endpoint: kmsKey.KmsAdapterUrl,
				}
				svc.KmsClients.Store(int32(ki.ChainInfoId), kmsClient)
			}

			// 如果本次查询数量小于 limit，则说明已经查询完所有数据，则退出循环
			if len(keyInfos) < 20 {
				logx.Info("success to init kmsClients")
				return
			}

			// 继续查询下一页
			offset += 20
		}
	}()
}
