/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package util defines some common function
package util

import (
	"bytes"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"regexp"
	"strings"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/model/table"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/key"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/pb-go/v2/accesscontrol"
	"github.com/tjfoc/gmsm/x509"
	"github.com/zeromicro/go-zero/core/logx"
)

// ConvertToLogFields 将 map 转成 logField
func ConvertToLogFields(fields map[string]interface{}) (res []logx.LogField) {
	for k, v := range fields {

		// logx.Field 本身就是 kv
		res = append(res, logx.Field(k, v))
	}

	return res
}

// ConvertToAccountData 将数据库返回结构转成接口 response 需要的结构
func ConvertToAccountData(infos []table.KeyInfo, authUsers []map[uint32]string) (res []*key.AccountData) {
	for _, k := range infos {

		// 把数据库中存储的字符串转成数组
		var accountTypes []string
		if k.AccountType != "" {
			accountTypes = strings.Split(k.AccountType, "&")
		}

		// 转成 pb 数据结构
		res = append(res, &keypb.AccountData{

			// 私钥自增id
			KeyId: uint32(k.ID),
			// 私钥模式
			KeyMode: k.KeyMode,
			// 链id
			ChainId: k.ChainId,
			// 子链id
			SubChainId: k.SubChainId,
			// 链名称
			ChainName: k.ChainName,
			// 账户地址
			AccountAddress: k.AccountAddress,
			// 账户身份
			AccountName: k.AccountName,
			// 账户类型
			AccountTypes: accountTypes,
			// 账户社会信用代码
			SocialCreditCode: k.AccountSocialCreditCode,
			// 账户授权使用的 userId
			//AuthUsers: authUsers[i],
			// 链上实名认证 id
			RegisterId: uint32(k.RegisterId),

			// response 返回的统一的 unix 时间戳，客户端根据灵活转成本地时区时间
			CreatedAt:   k.CreatedAt.Unix(),
			UpdatedAt:   k.UpdatedAt.Unix(),
			Did:         k.Did,
			ChainInfoId: int32(k.ChainInfoId),
		})
	}

	return res
}

// ConvertToSignResponseMember 将链上参数结构转成接口 response 需要的结构
func ConvertToSignResponseMember(m *accesscontrol.Member) *key.SignResponse_Member {
	return &keypb.SignResponse_Member{
		OrgId:      m.OrgId,
		MemberInfo: m.MemberInfo,
		MemberType: int32(m.MemberType),
	}
}

func ConvertPubKeyHexListToByteList(pubKeyHexList []string) ([]string, error) {
	var pubKeyStringList []string
	for _, pubKeyHex := range pubKeyHexList {
		pubKey, err := x509.ReadPublicKeyFromHex(pubKeyHex)
		if err != nil {
			return nil, err
		}
		pubKeyByte, err := x509.MarshalSm2PublicKey(pubKey)
		if err != nil {
			return nil, err
		}
		pubKeyStringList = append(pubKeyStringList, string(pubKeyByte))
	}
	return pubKeyStringList, nil
}

func ConvertBytesList2HexList(bytesList [][]byte) []string {
	var hexList []string
	for _, bytes := range bytesList {
		hexString := hex.EncodeToString(bytes)
		hexList = append(hexList, hexString)
	}
	return hexList
}

// IsAvailablePk 判断pk是否是hex
func IsAvailablePk(pk string) bool {
	re := regexp.MustCompile(`^[0-9a-fA-F]+$`)
	return re.MatchString(pk)
}

// SendHttpRequest 发送http请求
func SendHttpRequest(url, method string, params interface{}) ([]byte, error) {

	// 序列化请求参数
	paBytes, err := json.Marshal(params)
	if err != nil {
		return nil, fmt.Errorf("json marshal req params failed: %v", err)
	}

	// 创建请求
	req, err := http.NewRequest(method, url, bytes.NewBuffer(paBytes))
	if err != nil {
		return nil, fmt.Errorf("new http req failed: %v", err)
	}

	// 设置请求头
	req.Header.Set("Content-Type", "application/json")

	// 发送请求
	res, err := http.DefaultClient.Do(req)
	if err != nil {
		return nil, fmt.Errorf("send req failed: %v", err)
	}

	// 读取响应
	defer res.Body.Close()
	bBytes, err := io.ReadAll(res.Body)
	if err != nil {
		return nil, fmt.Errorf("read response body failed: %v", err)
	}

	return bBytes, nil
}
