/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package main 主程序入口
package main

import (
	"flag"
	"fmt"
	"os"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/config"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/server"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	commonGrpc "chainweaver.org.cn/chainweaver/ida/common/v2/grpc"
	miraConfig "chainweaver.org.cn/chainweaver/mira/mira-common/config"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"github.com/zeromicro/go-zero/core/conf"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/core/service"
	"github.com/zeromicro/go-zero/gateway"
	"google.golang.org/grpc"
	"google.golang.org/grpc/reflection"
)

var configFile = flag.String("f", "etc/key-service.yaml", "the config file")

func main() {
	flag.Parse()

	// version 选项打印当前版本信息
	args := flag.Args()
	if len(args) > 0 && args[0] == "version" {
		fmt.Println(internal.VersionInfo())
		os.Exit(0)
	}

	var c config.Config
	conf.MustLoad(*configFile, &c)
	ctx := svc.NewServiceContext(c)

	register := func(grpcServer *grpc.Server) {
		keypb.RegisterKeyServer(grpcServer, server.NewKeyServer(ctx))

		// 在 dev、test 环境开启调试功能
		if c.Mode == service.DevMode || c.Mode == service.TestMode {
			reflection.Register(grpcServer)
		}
	}

	miraDBConf := miraConfig.DBConfig{
		Type: c.DBConf.Type,
		URL:  c.DBConf.DNS,
	}
	err := dbaccess.InitDBAccessService(&miraDBConf)
	if err != nil {
		panic(fmt.Errorf("failed to database,error: %v", err))
	}
	logx.Infof("init db success!")

	s, err := commonGrpc.CreateGRPCServer(c.RpcServerConf, register, c.GrpcConf.CaCertFile, c.GrpcConf.ServerCertFile,
		c.GrpcConf.ServerKeyFile, c.GrpcConf.MaxRecvMsgSize, c.GrpcConf.MaxSendMsgSize)
	if err != nil {
		panic(fmt.Errorf("failed to CreateGRPCServer,error: %v", err))
	}

	gw := gateway.MustNewServer(c.Gateway)
	group := service.NewServiceGroup()
	group.Add(s)
	group.Add(gw)
	defer group.Stop()

	fmt.Printf("Starting rpc server at %s...\n", c.ListenOn)
	fmt.Printf("Starting gateway at %s:%d...\n", c.Gateway.Host, c.Gateway.Port)
	logx.Infof("Starting rpc server at %s...\n", c.ListenOn)
	logx.Infof("Starting gateway at %s:%d...\n", c.Gateway.Host, c.Gateway.Port)
	group.Start()
}
